<?php
/**
 * Plugin Name:       Email Cart for WooCommerce
 * Plugin URI:        http://emailcartplugin.com/
 * Description:       Populate a shopping cart on the front end of a WooCommerce store and email a link to a friend to restore the cart on their end.
 * Version:           1.4.6
 * Author:            Rymera Web Co
 * Author URI:        https://rymera.com.au/
 * Text Domain:       email-cart-for-woocommerce
 */

require_once ( 'email-cart-for-woocommerce.functions.php' );

// Delete code activation flag on plugin deactivate.
register_deactivation_hook( __FILE__, 'ecwGlobalPluginDeactivate' );

// Check if WooCommerce is active
if ( ecwWooCommercePluginActiveCheck() ) {

	// Include Necessary Files
	require_once ( 'email-cart-for-woocommerce.options.php' );
	require_once ( 'email-cart-for-woocommerce.plugin.php' );

	// Get Instance of Main Plugin Class
	$email_cart_for_woocommerce = Email_Cart_For_WooCommerce::getInstance();

	$GLOBALS[ 'email_cart_for_woocommerce' ] = $email_cart_for_woocommerce;

	// Register Activation Hook
	register_activation_hook( __FILE__, array( $email_cart_for_woocommerce, 'activate' ) );

	// Register Deactivation Hook
	register_deactivation_hook( __FILE__, array( $email_cart_for_woocommerce, 'deactivate' ) );

	// Plugin Initialization
	add_action( 'init', array( $email_cart_for_woocommerce, 'initialize' ) );

	//  Register AJAX Call Handlers
	add_action( 'init', array( $email_cart_for_woocommerce, 'registerAJAXCAllHandlers' ) );

	// Execute plugin initialization ( plugin activation ) on every newly created site in a multi site set up
    add_action( 'wpmu_new_blog', array( $email_cart_for_woocommerce, 'ecwMultisiteInit' ), 10, 6 );

	// Load Backend CSS and JS
	add_action( 'admin_enqueue_scripts', array( $email_cart_for_woocommerce, 'loadBackEndStylesAndScripts' ) );

	// Load Frontend CSS and JS
	add_action( 'wp_enqueue_scripts', array( $email_cart_for_woocommerce, 'loadFrontEndStylesAndScripts' ) );

	add_filter( 'woocommerce_get_settings_pages', array( $email_cart_for_woocommerce, 'ecwPluginSettings' ) );

	// Plugin action links
    add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $email_cart_for_woocommerce, 'ecwAddPluginListingActionLinks' ), 10, 2 );

	// Manage Shopping Cart on user Schedule - Using wp cron job
	add_action( ECW_CRON_HOOK, array( $email_cart_for_woocommerce, 'ecwRemoveExpiredCart'), 10, 3 );

	// Register new Shopping Cart CPT
	add_action( 'init', array( $email_cart_for_woocommerce, 'ecwRegisterShoppingCartCPT' ) );

	// Add custom column to Shopping Cart CPT
	add_filter( 'manage_shopping-cart_posts_columns', array( $email_cart_for_woocommerce, 'ecwSetNewShoppingCartColumn' ), 10, 1 );
	add_action( 'manage_shopping-cart_posts_custom_column' , array( $email_cart_for_woocommerce, 'ecwShoppingCartNewColumns' ), 10, 2 );
	add_action( 'admin_footer', array( $email_cart_for_woocommerce, 'ecwSetConfirmDeleteItemBoxHTML' ) );

	// Add Email Cart Button in Woocommerce Shopping cart page
	add_action( 'woocommerce_cart_actions', array( $email_cart_for_woocommerce, 'ecwAddEmailCartButtonInCartPage' ), 100 );
	add_action( 'wp_footer', array( $email_cart_for_woocommerce, 'ecwSetEmailCartFormModalBoxHTML' ) );

	// Populate cart with items sent by the sender.
	add_action( 'template_redirect', array( $email_cart_for_woocommerce, 'ecwPopulateCart' ), 10 );

	// Log user cart restores upon order process
	add_action(  'woocommerce_checkout_order_processed' , array( $email_cart_for_woocommerce , 'ecwLogUserCartRestore' ) , 10 , 1 );

	// Remove Yoast meta box and other meta box that is not needed, also remove all columns for wpseo in the list screen
	add_action( 'add_meta_boxes', array( $email_cart_for_woocommerce, 'ecwRemoveMetaBoxes' ), 20 );

	// Add new meta boxes
	add_action( 'add_meta_boxes', array( $email_cart_for_woocommerce, 'ecwShoppingCartMetaBoxes' ), 10 );

	// Load plugin text domain
	add_action( 'plugins_loaded', array( $email_cart_for_woocommerce, 'ecwLoadPluginTextdomain' ) );

	// Delete scheduled cart expiration cron event
	add_action( 'ecw_before_cart_delete', array( $email_cart_for_woocommerce, 'ecwDeleteScheduledCartExpirationCron' ) );
	add_action( 'delete_post', array( $email_cart_for_woocommerce, 'ecwDeleteScheduledCartExpirationCron' ) );

	// Check if Product Bundles plugin is activated
	if( ecwProductBundlePluginActiveCheck() ){

		// Display Bundled products properly on the ordered items meta box
		add_filter( 'ecw_cart_item_class', array( $email_cart_for_woocommerce, 'ecwBundlesTableItemClass' ), 10, 3 );
		add_filter( 'ecw_cart_item_price', array( $email_cart_for_woocommerce, 'ecwBundlesTableItemPrice' ), 10, 3 );
		add_filter( 'ecw_cart_item_quantity', array( $email_cart_for_woocommerce, 'ecwBundlesTableItemQuantity' ), 10, 3 );
		add_filter( 'ecw_cart_item_subtotal', array( $email_cart_for_woocommerce, 'ecwBundlesItemSubtotal' ), 10, 3 );

	}

	// Check if Composite Products plugin is activated
	if( ecwCompositeProductPluginActiveCheck() ){

		// Display Composite products properly on the ordered items meta box
		add_filter( 'ecw_cart_item_class', array( $email_cart_for_woocommerce, 'ecwCompositeTableItemClass' ), 10, 3 );
		add_filter( 'ecw_cart_item_name', array( $email_cart_for_woocommerce, 'ecwCompositeItemName' ), 10, 3 );
		add_filter( 'ecw_cart_item_price', array( $email_cart_for_woocommerce, 'ecwCompositeItemPrice' ), 10, 3 );
		add_filter( 'ecw_cart_item_subtotal', array( $email_cart_for_woocommerce, 'ecwCompositeItemSubtotal' ), 10, 3 );

	}

}else{

    /**
     * Display admin notice that WooCommerce is not installed.
     *
     * @since 1.3.0
     */
    function ecwAdminNotices() {

		// Indicate that activation callback isn't triggered
	    delete_option( 'ecw_activation_code_triggered' );

        $adminNoticeMsg = '';
        $pluginKey 		= 'woocommerce';
        $pluginName  	= 'WooCommerce';
        $pluginFile     = 'woocommerce/woocommerce.php';
        $sptFile        = trailingslashit( WP_PLUGIN_DIR ) . plugin_basename( $pluginFile );

        $sptInstallText = '<a href="' . wp_nonce_url( 'update.php?action=install-plugin&plugin=' . $pluginKey, 'install-plugin_' . $pluginKey ) . '">' . __( 'Click here to install from WordPress.org repo &rarr;', 'email-cart-for-woocommerce' ) . '</a>';
        if ( file_exists( $sptFile ) )
            $sptInstallText = '<a href="' . wp_nonce_url( 'plugins.php?action=activate&amp;plugin=' . $pluginFile . '&amp;plugin_status=all&amp;s', 'activate-plugin_' . $pluginFile ) . '" title="' . __( 'Activate this plugin', 'email-cart-for-woocommerce' ) . '" class="edit">' . __( 'Click here to activate &rarr;', 'email-cart-for-woocommerce' ) . '</a>';

        $adminNoticeMsg .= sprintf( __( '<br/>Please ensure you have <a href="%1$s" target="_blank">%2$s</a> plugin installed and activated.<br/>', 'email-cart-for-woocommerce' ), 'http://wordpress.org/plugins/' . $pluginKey. '/', $pluginName );
        $adminNoticeMsg .= $sptInstallText . '<br/>'; ?>

        <div class="error">
            <p>
                <?php _e( '<b>Email Cart for WooCommerce</b> plugin missing dependency.<br/>', 'email-cart-for-woocommerce' ); ?>
                <?php echo $adminNoticeMsg; ?>
            </p>
        </div><?php

    }

    add_action( 'admin_notices', 'ecwAdminNotices' );

}
