<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/*
 |--------------------------------------------------------------------------------------------------------------
 | MISC Functions
 |--------------------------------------------------------------------------------------------------------------
 */

/**
 * Delete code activation flag on plugini deactivate.
 *
 * @since 1.3.0
 */
if ( ! function_exists( 'ecwWooCommercePluginActiveCheck' ) ) {
	function ecwWooCommercePluginActiveCheck(){
		return in_array( 'woocommerce/woocommerce.php', get_option( 'active_plugins' ) );
	}
}

/**
 * Check if bundle plugin is active
 *
 * @since 1.2.4
 */
if ( ! function_exists( 'ecwProductBundlePluginActiveCheck' ) ) {
	function ecwProductBundlePluginActiveCheck(){
		return in_array( 'woocommerce-product-bundles/woocommerce-product-bundles.php', get_option( 'active_plugins' ) );
	}
}

/**
 * Check if Composite plugin is active
 *
 * @since 1.2.4
 */
if ( ! function_exists( 'ecwCompositeProductPluginActiveCheck' ) ) {
	function ecwCompositeProductPluginActiveCheck(){
		return in_array( 'woocommerce-composite-products/woocommerce-composite-products.php', get_option( 'active_plugins' ) );
	}
}

/**
 * Delete code activation flag on plugini deactivate.
 *
 * @since 1.3.0
 */
if( ! function_exists( 'ecwArrayInsertAfter' ) ){
	function ecwArrayInsertAfter( array $array, $key, array $new ) {
		$keys = array_keys( $array );
		$index = array_search( $key, $keys );
		$pos = false === $index ? count( $array ) : $index + 1;
		return array_merge( array_slice( $array, 0, $pos ), $new, array_slice( $array, $pos ) );
	}
}

/**
 * Delete code activation flag on plugin deactivate.
 *
 * @since 1.3.0
 */
if( ! function_exists( 'ecwGlobalPluginDeactivate' ) ){
	function ecwGlobalPluginDeactivate( $networkWide ) {

		global $wpdb;

		// check if it is a multisite network
		if ( is_multisite() ) {

			// check if the plugin has been deactivated on the network or on a single site
			if ( $networkWide ) {

				// get ids of all sites
				$blogIDs = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );

				foreach ( $blogIDs as $blogID ) {

					switch_to_blog( $blogID );
					delete_option( 'ecw_activation_code_triggered' );

				}

				restore_current_blog();

			} else {

				// deactivated on a single site, in a multi-site
				delete_option( 'ecw_activation_code_triggered' );

			}

		} else {

			// deactivated on a single site
			delete_option( 'ecw_activation_code_triggered' );

		}
	}
}