<?php
if ( !defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( !class_exists( 'Email_Cart_For_WooCommerce' ) ) {

    /**
     * Include Classes
     */
    if( !class_exists( 'ECW_Shopping_Cart' ) )
        require_once ( ECW_INCLUDES_DIR . 'class-ecw-shopping-cart.php' );

    if( !class_exists( 'ECW_Custom_Meta_Boxes' ) )
        require_once ( ECW_INCLUDES_DIR . 'class-ecw-custom-meta-boxes.php' );

    if( !class_exists( 'ECW_Emails' ) )
        require_once ( ECW_INCLUDES_DIR . 'class-ecw-emails.php' );

    if( !class_exists( 'ECW_Cron' ) )
        require_once ( ECW_INCLUDES_DIR . 'class-ecw-cron.php' );

    if( !class_exists( 'ECW_AJAX' ) )
        require_once ( ECW_INCLUDES_DIR . 'class-ecw-ajax.php' );

    if( !class_exists( 'ECW_Bundled_Products' ) )
        require_once ( ECW_INCLUDES_DIR . 'class-ecw-bundled-products.php' );

    if( !class_exists( 'ECW_Composite_Products' ) )
        require_once ( ECW_INCLUDES_DIR . 'class-ecw-composite-products.php' );

    if( !class_exists( 'ECW_Guided_Tour' ) )
        require_once ( ECW_INCLUDES_DIR . 'class-ecw-guided-tour.php' );

    if( !class_exists( 'ECW_HELPER' ) )
        require_once ( ECW_INCLUDES_DIR . 'class-ecw-helper.php' );

    /**
     * Class Email_Cart_For_WooCommerce
     */
    class Email_Cart_For_WooCommerce {

        /*
         |--------------------------------------------------------------------------------------------------------------
         | Class Members
         |--------------------------------------------------------------------------------------------------------------
         */

        private static $_instance;

        private $_ecw_shopping_cart;
        private $_ecw_custom_meta_boxes;
        private $_ecw_emails;
        private $_ecw_cron;
        private $_ecw_ajax;
        private $_ecw_bundled_products;
        private $_ecw_composite_products;
        private $_ecw_guided_tour;

        const VERSION = '1.4.6';

        /*
         |--------------------------------------------------------------------------------------------------------------
         | Mesc Functions
         |--------------------------------------------------------------------------------------------------------------
         */

        /**
         * Class constructor.
         *
         * @since 1.0.0
         */
        public function __construct() {

            $this->_ecw_shopping_cart = ECW_Shopping_Cart::getInstance();
            $this->_ecw_custom_meta_boxes = ECW_Custom_Meta_Boxes::getInstance();
            $this->_ecw_emails = ECW_Emails::getInstance();
            $this->_ecw_cron = ECW_Cron::getInstance();
            $this->_ecw_ajax = ECW_AJAX::getInstance();
            $this->_ecw_bundled_products = ECW_Bundled_Products::getInstance();
            $this->_ecw_composite_products = ECW_Composite_Products::getInstance();
            $this->_ecw_guided_tour = ECW_Guided_Tour::getInstance();

        }

        /**
         * Create plugin instance.
         *
         * @return Woocommerce_Email_Cart
         * @since 1.0.0
         */
        public static function getInstance() {

            if( !self::$_instance instanceof self )
                self::$_instance = new self;

            return self::$_instance;

        }

        /*
         |--------------------------------------------------------------------------------------------------------------
         | Bootstrap/Shutdown Functions
         |--------------------------------------------------------------------------------------------------------------
         */

        /**
         * Method to initialize a newly created site in a multi site set up.
         *
         * @param $blogID
         * @param $userID
         * @param $domain
         * @param $path
         * @param $siteID
         * @param $meta
         *
         * @since 1.3.0
         */
        public function ecwMultisiteInit( $blogID, $userID, $domain, $path, $siteID, $meta ) {

            if ( is_plugin_active_for_network( 'email-cart-for-woocommerce/email-cart-for-woocommerce.bootstrap.php' ) ) {

                switch_to_blog( $blogID );
                $this->ecwInitDefaultData( $blogID );
                restore_current_blog();

            }
        }

        /**
         * Plugin initialization.
         *
         * @since 1.0.0
         */
        public function initialize() {

            $activationFlag = get_option( ECW_ACTIVATION_CODE_TRIGGERED, false );

            if ( $activationFlag != 'yes' ){

                if ( ! function_exists( 'is_plugin_active_for_network' ) )
                    require_once( ABSPATH . '/wp-admin/includes/plugin.php' );

                $networkWide = is_plugin_active_for_network( 'email-cart-for-woocommerce/email-cart-for-woocommerce.bootstrap.php' );

                $this->activate( $networkWide );

            }

        }

        /**
         * Plugin activation hook callback.
         *
         * @since 1.0.0
         */
        public function activate( $networkWide ) {

            global $wpdb;

            if( is_multisite() ){

                if( $networkWide ){

                    // get ids of all sites
                    $blogIDs = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );

                    foreach( $blogIDs as $blogID ){

                        switch_to_blog( $blogID );
                        $this->ecwInitDefaultData( $blogID );

                    }

                    restore_current_blog();

                }else{

                    // activated on a single site, in a multi-site
                    $this->ecwInitDefaultData( $wpdb->blogid );

                }

            }else{

                // activated on a single site
                $this->ecwInitDefaultData( $wpdb->blogid );

            }

            update_option( ECW_ACTIVATION_CODE_TRIGGERED, 'yes' );

        }

        /**
         * Actual function that houses the code to execute on plugin activation.
         *
         * @param $blogID
         *
         * @since 1.3.0
         */
        private function ecwInitDefaultData( $blogID ) {

            // Add Shopping Cart CPT
            if ( ! post_type_exists( ECW_CPT_NAME ) )
                $this->_ecw_shopping_cart->ecwRegisterShoppingCartCPT();

            // Guided Tour
            if ( get_option( 'ecw_guided_tour_status' ) === false )
                update_option( 'ecw_guided_tour_status', 'open' );

            flush_rewrite_rules();

        }

        /**
         * Plugin deactivation hook callback.
         *
         * @since 1.0.0
         */
        public function deactivate( $networkWide ) {

            global $wpdb;

            // check if it is a multisite network
            if ( is_multisite() ) {

                // check if the plugin has been activated on the network or on a single site
                if ( $networkWide ) {

                    // get ids of all sites
                    $blogIDs = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );

                    foreach ( $blogIDs as $blogID ) {

                        switch_to_blog( $blogID );
                        $this->ecwDeactivateActions( $blogID );

                    }

                    restore_current_blog();

                } else {

                    // activated on a single site, in a multi-site
                    $this->ecwDeactivateActions( $wpdb->blogid );

                }

            } else {

                // activated on a single site
                $this->ecwDeactivateActions( $wpdb->blogid );

            }

        }

        /**
         * Perform deactivation actions.
         *
         * @param $blogID
         *
         * @since 1.3.0
         */
        private function ecwDeactivateActions( $blogID ){

            flush_rewrite_rules();

        }

        /*
        |---------------------------------------------------------------------------------------------------------------
        | Settings
        |---------------------------------------------------------------------------------------------------------------
        */

        /**
         * Plugin action links
         *
         * @param array $links
         * @param string $file
         *
         * @return array
         * @since 1.3.0
         */
        public function ecwAddPluginListingActionLinks( $links, $file ){

            $help       = '<a target="_blank" href="https://marketingsuiteplugin.com/knowledge-base/email-cart/?utm_source=ECW&utm_medium=Settings%20Help&utm_campaign=ECW">' . __( 'Help' , 'email-cart-for-woocommerce' ) . '</a>';
            $settings   = '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=ecw_settings' ) . '">' . __( 'Settings' , 'email-cart-for-woocommerce' ) . '</a>';
            array_unshift( $links , $help , $settings );

            return apply_filters( 'ecw_plugin_listings_action_links', $links, $file, $settings );

        }

        /**
         * Initialize plugin settings.
         *
         * @param $settings
         *
         * @return array
         * @since 1.3.0
         */
        public function ecwPluginSettings( $settings ) {

            $settings[] = include( ECW_INCLUDES_DIR . 'class-ecw-settings.php' );

            return $settings;

        }

        /*
        |---------------------------------------------------------------------------------------------------------------
        | Admin Functions
        |---------------------------------------------------------------------------------------------------------------
        */

        /**
         * Load admin or back-end related styles and scripts.
         *
         * @since 1.0.0
         */
        public function loadBackEndStylesAndScripts() {

            $screen = get_current_screen();

            if( is_admin() && $screen->post_type == ECW_CPT_NAME ){

                // Styles
                wp_enqueue_style( 'ecw_shopping_cart_back_end_css', ECW_CSS_URL . 'ecw-back-end-shopping-cart.css', array(), self::VERSION, 'all' );
                wp_enqueue_style ( 'wp-jquery-ui-dialog' );
                wp_enqueue_style( 'ecw_toastr_css', ECW_JS_URL . 'lib/toastr/toastr.min.css', array(), self::VERSION , 'all' );

                // Scripts
                wp_enqueue_script( 'ecw_toastr_js', ECW_JS_URL . 'lib/toastr/toastr.min.js', array( 'jquery' ), self::VERSION );
                wp_enqueue_script( 'ecw_back_end_ajax_services_js', ECW_JS_URL . 'app/modules/BackEndAjaxServices.js', array( 'jquery' ), self::VERSION, true );
                wp_enqueue_script( 'ecw_shopping_cart_actions_js', ECW_JS_URL . 'app/modules/ShoppingCartActions.js', array( 'jquery' ), self::VERSION, true );
                wp_enqueue_script( 'ecw_shopping_cart_js', ECW_JS_URL . 'app/ecw-shopping-cart.js', array( 'jquery' ), self::VERSION, true );
                wp_localize_script( 'ecw_shopping_cart_js',
                                    'ecw_email_cart_js_vars', apply_filters( 'ecw_email_cart_js_vars',
                                                                array(
                                                                    'dialog_delete_warning'     => __( 'Warning!', 'email-cart-for-woocommerce' ),
                                                                    'dialog_delete_yes_btn'     => __( 'Yes', 'email-cart-for-woocommerce' ),
                                                                    'dialog_delete_no_btn'      => __( 'No', 'email-cart-for-woocommerce' ),
                                                                ) ) );
                wp_enqueue_script( 'ecw_clipboard_js', ECW_JS_URL . 'lib/clipboardjs/clipboard.min.js', array( 'jquery' ), self::VERSION, true );
                wp_enqueue_script( 'ecw_clipboard_action_js', ECW_JS_URL . 'app/ecw-clipboard.js', array( 'jquery' ), self::VERSION, true );
                wp_localize_script( 'ecw_clipboard_action_js',
                                    'ecw_clipboard_action_js', apply_filters( 'ecw_clipboard_action_js',
                                                                array(
                                                                    'success_msg'   => __( 'Copied to clipboard', 'email-cart-for-woocommerce' ),
                                                                    'error_msg'     => __( 'Error', 'email-cart-for-woocommerce' ),
                                                                ) ) );
                wp_enqueue_script( 'jquery-ui-dialog' );

            }

            // Guided Tours
            if ( get_option( 'ecw_guided_tour_status', false ) == 'open' && array_key_exists( $screen->id, $this->_ecw_guided_tour->ecwGetScreens() ) ) {
                wp_enqueue_style( 'ecw_plugin-guided-tour_css', ECW_CSS_URL . 'ecw-guided-tour.css', array( 'wp-pointer' ), self::VERSION, 'all' );
                wp_enqueue_script( 'ecw_plugin-guided-tour_js', ECW_JS_URL . 'app/ecw-guided-tour.js', array( 'wp-pointer', 'thickbox' ), self::VERSION, true );

                wp_localize_script( 'ecw_plugin-guided-tour_js',
                                    'ecw_guided_tour_params',
                                    array(
                                        'actions' => array( 'close_tour' => 'ecwCloseGuidedTour' ),
                                        'nonces'  => array( 'close_tour' => wp_create_nonce( 'ecw-close-guided-tour' ) ),
                                        'screen'  => $this->_ecw_guided_tour->ecwGetCurrentScreen(),
                                        'height'  => 640,
                                        'width'   => 640,
                                        'texts'   => array(
                                                        'btn_prev_tour'  => __( 'Previous', 'email-cart-for-woocommerce' ),
                                                        'btn_next_tour'  => __( 'Next', 'email-cart-for-woocommerce' ),
                                                        'btn_close_tour' => __( 'Close', 'email-cart-for-woocommerce' ),
                                                        'btn_start_tour' => __( 'Start Tour', 'email-cart-for-woocommerce' )
                                                    ),
                                        'urls'    => array( 'ajax' => admin_url( 'admin-ajax.php' ) ),
                                        'post'    => isset( $post ) && isset( $post->ID ) ? $post->ID : 0
                                    ));

            }

        }

        /**
         * Load front-end related styles and scripts.
         *
         * @since 1.0.0
         */
        public function loadFrontEndStylesAndScripts() {

            if( is_cart() ){

                // Styles
                wp_enqueue_style( 'ecw_shopping_cart_front_end_css', ECW_CSS_URL . 'ecw-front-end-shopping-cart.css', array(), self::VERSION, 'all' );
                wp_enqueue_style ( 'wp-jquery-ui-dialog' );
                wp_enqueue_style( 'ecw_toastr_css', ECW_JS_URL . 'lib/toastr/toastr.min.css', array(), self::VERSION , 'all' );

                // Inline styles. Only add when ECWP frotend css is not enqueued
                if ( ! wp_style_is( 'ecwp_shopping_cart_front_end_css' ) ) {

                    $inline_styles = '
                        div.email-cart div.ui-dialog-titlebar,
                        div.email-cart #email-cart-form {
                            width: 100% !important;
                        }
                    ';
                    wp_add_inline_style( 'ecw_shopping_cart_front_end_css', $inline_styles );
                }


                // Scripts
                wp_enqueue_script( 'ecw_toastr_js', ECW_JS_URL . 'lib/toastr/toastr.min.js', array( 'jquery' ), self::VERSION );
                wp_enqueue_script( 'ecw_back_end_ajax_services_js', ECW_JS_URL . 'app/modules/BackEndAjaxServices.js', array( 'jquery' ) , self::VERSION, true );
                wp_localize_script( 'ecw_back_end_ajax_services_js', 'front_end_ajax_object', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
                wp_enqueue_script( 'ecw_shopping_cart_actions_js', ECW_JS_URL . 'app/modules/ShoppingCartActions.js', array( 'jquery' ), self::VERSION, true );
                wp_enqueue_script( 'ecw_shopping_cart_js', ECW_JS_URL . 'app/ecw-shopping-cart.js', array( 'jquery' ), self::VERSION, true );
                wp_localize_script( 'ecw_shopping_cart_js',
                                    'ecw_email_cart_js_vars', apply_filters( 'ecw_email_cart_js_vars',
                                                                array(
                                                                    'dialog_title'              => __( 'Email This Shopping Cart!', 'email-cart-for-woocommerce' ),
                                                                    'dialog_button_email_cart'  => __( 'Email Cart', 'email-cart-for-woocommerce' ),
                                                                    'sender_error_msg'          => __( '"Your Name" is a required field', 'email-cart-for-woocommerce' ),
                                                                    'recipient_error_msg'       => __( '"Recipient Name" is a required field', 'email-cart-for-woocommerce' ),
                                                                    'email_error_msg'           => __( 'Please enter a valid email address', 'email-cart-for-woocommerce' ),
                                                                    'general_error_msg'         => __( 'Error:', 'email-cart-for-woocommerce' )
                                                                ) ) );
                wp_enqueue_script( 'jquery-ui-dialog' );

            }

        }

        /**
         * Load plugin text domain
         *
         * @since 1.1.0
         */
        public function ecwLoadPluginTextdomain(){

            load_plugin_textdomain( 'email-cart-for-woocommerce', FALSE, basename( dirname( __FILE__ ) ) . '/languages/' );

        }

        /*
        |---------------------------------------------------------------------------------------------------------------
        | AJAX Callbacks
        |---------------------------------------------------------------------------------------------------------------
        */

        /**
         * Register AJAX callbacks.
         *
         * @since 1.0.0
         */
        public function registerAJAXCAllHandlers() {

            add_action( 'wp_ajax_ecwDeleteShoppingCartItem', array( self::getInstance(), 'ecwDeleteShoppingCartItem' ) );

            add_action( 'wp_ajax_ecwSendEmailCartToRecipient', array( self::getInstance(), 'ecwSendEmailCartToRecipient' ) );
            add_action( 'wp_ajax_nopriv_ecwSendEmailCartToRecipient', array( self::getInstance(), 'ecwSendEmailCartToRecipient' ) );

            // Plugin Guided Tours
            add_action( 'wp_ajax_ecwCloseGuidedTour', array( $this->_ecw_guided_tour, 'ecwCloseGuidedTour' ) );

        }

        /*
        |-------------------------------------------------------------------------------------------------------------------
        | Email Shopping Cart
        |-------------------------------------------------------------------------------------------------------------------
        */

        /**
         * Adds email cart button in woocommerce cart page
         *
         * @since 1.0.0
         */
        public function ecwAddEmailCartButtonInCartPage(){

            $this->_ecw_emails->ecwAddEmailCartButtonInCartPage();

        }

        /**
         * Sets the HTML email cart modal box
         *
         * @since 1.0.0
         */
        public function ecwSetEmailCartFormModalBoxHTML(){

            $this->_ecw_emails->ecwSetEmailCartFormModalBoxHTML();

        }

        /*
        |-------------------------------------------------------------------------------------------------------------------
        | Manage Shopping Cart
        |-------------------------------------------------------------------------------------------------------------------
        */

        /**
         * Delete shopping cart item.
         *
         * @param int $cartID
         *
         * @since 1.0.0
         */
        public function ecwDeleteShoppingCartItem( $cartID = null ){

            $this->_ecw_ajax->ecwDeleteShoppingCartItem( $cartID );

        }

        /**
         * Send the cart to the recipient.
         *
         * @param string $senderName
         * @param string $recipientName
         * @param string $recipientEmail
         *
         * @since 1.0.0
         */
        public function ecwSendEmailCartToRecipient( $senderName = null, $recipientName = null, $recipientEmail = null ){

            $this->_ecw_ajax->ecwSendEmailCartToRecipient( $senderName, $recipientName, $recipientEmail );

        }

        /*
        |-------------------------------------------------------------------------------------------------------------------
        | Shopping Cart Class
        |-------------------------------------------------------------------------------------------------------------------
        */

        /**
         * Used to register new Shopping Cart CPT
         *
         * @since 1.0.0
         */
        public function ecwRegisterShoppingCartCPT(){

            $this->_ecw_shopping_cart->ecwRegisterShoppingCartCPT();

        }

        /**
         * Create new column on the shopping cart CPT
         *
         * @param array $columns
         *
         * @return array
         * @since 1.0.0
         */
        public function ecwSetNewShoppingCartColumn( $columns ){

            return $this->_ecw_shopping_cart->ecwSetNewShoppingCartColumn( $columns );

        }

        /**
         * Sets the row value of the new column
         *
         * @param string $column
         * @param int $cartID
         *
         * @since 1.0.0
         */
        public function ecwShoppingCartNewColumns( $column, $cartID ){

            $this->_ecw_shopping_cart->ecwShoppingCartNewColumns( $column, $cartID );

        }

        /**
         * Set the HTML delete confirmation box
         *
         * @since 1.0.0
         */
        public function ecwSetConfirmDeleteItemBoxHTML(){

            $this->_ecw_shopping_cart->ecwSetConfirmDeleteItemBoxHTML();

        }

        /**
         * Populate shopping cart with data using the link sent by the sender.
         *
         * @since 1.0.0
         */
        public function ecwPopulateCart(){

            $this->_ecw_shopping_cart->ecwPopulateCart();

        }

        /**
         * Instantiate WC session, cart and customer
         *
         * @since 1.0.0
         */
        public function ecwWooCommerceInit(){

            $this->_ecw_shopping_cart->ecwWooCommerceInit();

        }

        /**
         * Set cart object items
         *
         * @since 1.0.0
         */
        public function ecwSetCartItems( $cid ){

            $this->_ecw_shopping_cart->ecwSetCartItems( $cid );

        }

        /**
         * Log user cart restores upon order process
         *
         * @param int $orderID  WC_Order object ID
         *
         * @since 1.4.0
         */
        public function ecwLogUserCartRestore( $orderID ){

            $this->_ecw_shopping_cart->ecwLogUserCartRestore( $orderID );

        }

        /**
         * save emal cart form information to database
         *
         * @param array $response   ecw_send_email response details
         *
         * @since 1.4.0
         */
        public function ecwSaveCartFormInfo( $response ) {

            $this->_ecw_shopping_cart->ecwSaveCartFormInfo( $response );
        }

        /*
        |---------------------------------------------------------------------------------------------------------------
        | Custom Meta Boxes Class
        |---------------------------------------------------------------------------------------------------------------
        */

        /**
         * Remove Yoast SEO meta box and other meta box that is not needed
         *
         * @since 1.0.0
         */
        public function ecwRemoveMetaBoxes(){

            $this->_ecw_custom_meta_boxes->ecwRemoveMetaBoxes();

        }

        /**
         * House all new meta boxes
         *
         * @since 1.0.0
         */
        public function ecwShoppingCartMetaBoxes(){

            $this->_ecw_custom_meta_boxes->ecwShoppingCartMetaBoxes();

        }

        /*
        |---------------------------------------------------------------------------------------------------------------
        | Cron Class
        |---------------------------------------------------------------------------------------------------------------
        */

        /**
         * Delete scheduled cart expiration cron event
         *
         * @param int $cartID
         *
         * @since 1.1.0
         */
        public function ecwDeleteScheduledCartExpirationCron( $cartID ){

            $this->_ecw_cron->ecwDeleteScheduledCartExpirationCron( $cartID );

        }

        /**
         * Remove expired cart
         *
         * @param int $cartID
         * @param string $cartStatus
         * @param string $cartPostType
         *
         * @since 1.1.0
         */
        public function ecwRemoveExpiredCart( $cartID, $cartStatus, $cartPostType ){

            $this->_ecw_cron->ecwRemoveExpiredCart( $cartID, $cartStatus, $cartPostType );

        }

        /*
        |---------------------------------------------------------------------------------------------------------------
        | Bundled Products Class
        |---------------------------------------------------------------------------------------------------------------
        */

        /**
         * Change the tr class of bundled items to allow their styling.
         *
         * @param string $classname
         * @param array $item
         * @param string $itemKey
         *
         * @return string
         * @since 1.2.0
         */
        public function ecwBundlesTableItemClass( $classname, $item, $itemKey ){

            return $this->_ecw_bundled_products->ecwBundlesTableItemClass( $classname, $item, $itemKey );

        }

        /**
         * Modify the edit screen product listings price of bundled items.
         *
         * @param string $price
         * @param array $item
         * @param string $itemKey
         *
         * @return string
         * @since 1.2.0
         */
        public function ecwBundlesTableItemPrice( $price, $item, $itemKey ){

            return $this->_ecw_bundled_products->ecwBundlesTableItemPrice( $price, $item, $itemKey );

        }

        /**
         * Modify the edit screen product listings subtotal of bundled items.
         *
         * @param string $subtotal
         * @param array $item
         * @param string $itemKey
         *
         * @return string
         * @since 1.2.0
         */
        public function ecwBundlesItemSubtotal( $subtotal, $item, $itemKey ){

            return $this->_ecw_bundled_products->ecwBundlesItemSubtotal( $subtotal, $item, $itemKey );

        }

        /**
         * Adjust quantity to follow product bundle rules.
         *
         * @param int $quantity
         * @param array $item
         * @param string $itemKey
         *
         * @return int
         * @since 1.2.0
         */
        public function ecwBundlesTableItemQuantity( $quantity, $item, $itemKey ){

            return $this->_ecw_bundled_products->ecwBundlesTableItemQuantity( $quantity, $item, $itemKey );

        }

        /*
        |---------------------------------------------------------------------------------------------------------------
        | Composite Products Class
        |---------------------------------------------------------------------------------------------------------------
        */

        /**
         * Change the tr class of composite items to allow their styling.
         *
         * @param string $classname
         * @param array $item
         * @param string $itemKey
         *
         * @return string
         * @since 1.2.0
         */
        public function ecwCompositeTableItemClass( $classname, $item, $itemKey ){

            return $this->_ecw_composite_products->ecwCompositeTableItemClass( $classname, $item, $itemKey );

        }

        /**
         * Adds order item title preambles to cart items ( Composite Attribute Descriptions )..
         *
         * @param string $name
         * @param array $item
         * @param string $itemKey
         *
         * @return string
         * @since 1.2.0
         */
        public function ecwCompositeItemName( $name, $item, $itemKey ){

            return $this->_ecw_composite_products->ecwCompositeItemName( $name, $item, $itemKey );

        }

        /**
         * Modify the edit screen product listings price of composite items.
         *
         * @param string $price
         * @param array $item
         * @param string $itemKey
         *
         * @return string
         * @since 1.2.0
         */
        public function ecwCompositeItemPrice( $price, $item, $itemKey ){

            return $this->_ecw_composite_products->ecwCompositeItemPrice( $price, $item, $itemKey );

        }

        /**
         * Modify the edit screen product listings subtotal of composite items.
         *
         * @param string $subtotal
         * @param array $item
         * @param string $itemKey
         *
         * @return string
         * @since 1.2.0
         */
        public function ecwCompositeItemSubtotal( $subtotal, $item, $itemKey ){

            return $this->_ecw_composite_products->ecwCompositeItemSubtotal( $subtotal, $item, $itemKey );

        }

        /**
         * Modify the edit screen product listings subtotal of composite items.
         *
         * @param int $qty
         * @param array $item
         * @param string $itemKey
         *
         * @return string
         * @since 1.2.0
         */
        public function ecwCompositeItemQuantity( $qty, $item, $itemKey ){

            return $this->_ecw_composite_products->ecwCompositeItemQuantity( $qty, $item, $itemKey );

        }

    }
}
