<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

class ECW_AJAX {

    private static $_instance;

    public static function getInstance(){
        if(!self::$_instance instanceof self)
            self::$_instance = new self;
        return self::$_instance;
    }

    /**
     * Delete Shopping Cart Item
     *
     * @param int $cartID
     *
     * @since 1.0.0
     */
    public function ecwDeleteShoppingCartItem( $cartID = null ){

        $cartID = defined( 'DOING_AJAX' ) && DOING_AJAX ? $_POST[ 'cartID' ] : $cartID;
        $cartID = (int) sanitize_text_field( $cartID );

        do_action( 'ecw_before_cart_delete', $cartID );

        if( ! empty( $cartID ) && is_int( $cartID ) ){

            // Delete Shopping Cart
            wp_delete_post( $cartID, true );

            $response = array(
                                'status'    => 'success',
                                'msg'       => __( 'Deleted Succesfully!', 'email-cart-for-woocommerce' )
                            );

        }else{

            $response = array(
                                'status'    => 'error',
                                'msg'       => __( 'Unable to delete', 'email-cart-for-woocommerce' )
                            );

        }

        do_action( 'ecw_after_cart_delete', $cartID );

        if( defined( 'DOING_AJAX' ) && DOING_AJAX ){

            header( 'Content-Type: application/json' );
            echo json_encode( $response );
            die();

        }else{

            return $response;

        }
    }

    /**
     * Send the cart to the recipient.
     *
     * @param mixed $cid
     * @param string $sendersName
     * @param string $recipientName
     * @param string $recipientEmail
     * @param string $nonce
     *
     * @since 1.0.0
     * @since 1.4.5 Added nonce to verify if the request came from a valid source also to avoid any spam request.
     */
    public function ecwSendEmailCartToRecipient( $cid = null, $sendersName = null, $recipientName = null, $recipientEmail = null, $nonce = null ){

        if( defined( 'DOING_AJAX' ) && DOING_AJAX ){
            $cID            = isset( $_POST[ 'cid' ] ) ? (int) sanitize_text_field( $_POST[ 'cid' ] ) : '';
            $sendersName    = isset( $_POST[ 'sendersName' ] ) ? sanitize_text_field( $_POST[ 'sendersName' ] ) : '';
            $recipientName  = isset( $_POST[ 'recipientName' ] ) ? sanitize_text_field( $_POST[ 'recipientName' ] ) : '';
            $recipientEmail = isset( $_POST[ 'recipientEmail' ] ) ? sanitize_text_field( $_POST[ 'recipientEmail' ] ) : '';
            $nonce          = isset( $_POST[ 'nonce' ] ) ? sanitize_text_field( $_POST[ 'nonce' ] ) : '';
        }

        // Verify Nonce
        if( isset( $nonce ) && check_ajax_referer( 'email-cart-send' , 'nonce' ) && ! empty( WC()->cart ) ) {

            // Get instance
            $ecwEmail       = ECW_Emails::getInstance();
            $ecwCart        = ECW_Shopping_Cart::getInstance();
            $wcEmails       = WC_Emails::instance();

            $cID            = (int) sanitize_text_field( $cid );
            $sendersName    = sanitize_text_field( $sendersName );
            $recipientName  = sanitize_text_field( $recipientName );
            $recipientEmail = sanitize_text_field( $recipientEmail );

            $cartID         = $ecwCart->ecwCreateCartLinkEntry( $cID );
            $website        = site_url();

            $hashedCartID   = is_numeric( $cartID ) ? md5( $cartID ) : $cartID;
            $link           = trailingslashit( wc_get_cart_url() );
            $link           = $link . '?cid=' . $hashedCartID;

            $fromName       = $ecwEmail->ecwWPMailFromName( $sendersName );
            $fromEmail      = $ecwEmail->ecwWPMailFrom();

            // Subject
            $subject = apply_filters( 'ecw_email_subject', 'Shopping Cart' );

            // Body
            $body = '<p>Hi ' . $recipientName . ',</p>';
            $body .= '<p>' . $sendersName . ' has sent you a pre-filled shopping cart on <a href="' . $website . '">' . $website . '</a>.</p>';
            $body .= '<p>Click here to view the shopping cart contents and make your purchase:</p>';
            $body .= '<p><a href="' . $link . '">' . $link . '</a></p>';
            $body .= '<p>Kind Regards,<br/>';
            $body .= '<a href="' . $website . '">' . $website . '</a><br/>';
            $body .= '<br/><br/><br/><br/>';
            $body = apply_filters( 'ecw_email_body', $body );

            // Headers
            $headers    = $ecwEmail->ecwConstructEmailHeader( $fromName, $fromEmail );
            $to         = array( $recipientName . ' <' . $recipientEmail . '> ' );

            // Send and update status if success
            $isSent = $wcEmails->send( $to, $subject, $body, $headers );

            if( $isSent === true ){

                $response = array(  'status'            => 'success',
                                    'cartID'            => $cartID,
                                    'link'              => $link,
                                    'body'              => $body,
                                    'sendersName'       => $sendersName,
                                    'recipientName'     => $recipientName,
                                    'recipientEmail'    => $recipientEmail,
                                    'subject'           => $subject,
                                    'body'              => $body,
                                    'message'           => '',
                                    'headers'           => $headers
                                );

            }else{

                $response = array(  'status'    => 'error',
                                    'msg'       => __( 'Email not sent!', 'email-cart-for-woocommerce' )
                                );

            }

            // save emal cart form information to database
            $ecwCart->ecwSaveCartFormInfo( $response );

            do_action( 'ecw_send_email', $isSent , $response );

        }else{

            $response = array(  'status'    => 'error',
                                'msg'       => __( 'Invalid action!', 'email-cart-for-woocommerce' )
                            );

        }

        if( defined( 'DOING_AJAX' ) && DOING_AJAX ){

            header( 'Content-Type: application/json' );
            echo json_encode( $response );
            die();

        }else{

            return $response;

        }
    }
}
