<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

class ECW_Bundled_Products {

    private static $_instance;
    private $enqueued_bundled_table_item_js = false;

    public static function getInstance(){
        if(!self::$_instance instanceof self)
            self::$_instance = new self;
        return self::$_instance;
    }

    /**
     * Enqeue js that wraps bundled table items in a div in order to apply indentation reliably.
     *
     * @return void
     * @since 1.2.0
     */
    private function ecwBundlesEnqueueBundledTableItemJS() {

        if ( ! $this->enqueued_bundled_table_item_js ) {
            wc_enqueue_js( "
                var ecw_pb_wrap_bundled_table_item = function() {
                    jQuery( '.bundled_table_item td.product-name' ).wrapInner( '<div class=\"bundled_table_item_indent\"></div>' );
                };

                ecw_pb_wrap_bundled_table_item();
            " );

            $this->enqueued_bundled_table_item_js = true;
        }
    }

    /**
     * Change the tr class of bundled items to allow their styling.
     *
     * @param string $classname
     * @param array $item
     * @param string $itemKey
     *
     * @return string
     * @since 1.2.0
     */
    public function ecwBundlesTableItemClass( $classname, $item, $itemKey ) {

        if ( isset( $item[ 'bundled_by' ] ) ){

            $this->ecwBundlesEnqueueBundledTableItemJS();

            return $classname . ' bundled_table_item';

        } elseif ( isset( $item[ 'stamp' ] ) )
            return $classname . ' bundle_table_item';

        return $classname;

    }

    /**
     * Modify the edit screen product listings price of bundled items and container items depending on the bundles's pricing strategy.
     *
     * @param string $price
     * @param array $item
     * @param string $itemKey
     *
     * @return string
     * @since 1.2.0
     * @since 1.2.4 On Product Bundles v1.5.0, woo_bundles_cart_item_price_html function is now deprecated. Used cart_item_price_html function instead.
     */
    public function ecwBundlesTableItemPrice( $price, $item, $itemKey ){

        if( class_exists( 'WC_PB_Cart') ){

            $wcPBCart = WC_PB_Cart::instance();
            return $wcPBCart->cart_item_price_html( $price, $item, $itemKey );

        }else
            return $price;

    }

    /**
     * Adjust quantity to follow product bundle rules.
     *
     * @param int $quantity
     * @param array $item
     * @param string $itemKey
     *
     * @return int
     * @since 1.2.0
     * @since 1.2.4 On Product Bundles v1.5.0, woo_bundles_cart_item_quantity function is now deprecated. Used cart_item_quantity function instead.
     */
    public function ecwBundlesTableItemQuantity( $quantity, $item, $itemKey ){

        if( class_exists( 'WC_PB_Cart') ){

            $wcPBCart = WC_PB_Cart::instance();
            return $wcPBCart->cart_item_quantity( $quantity, $itemKey );

        }else
            return $quantity;

    }

    /**
     * Modify the edit screen product listings subtotal of bundled items and container items depending on the bundles's pricing strategy.
     *
     * @param string $subtotal
     * @param array $item
     * @param string $itemKey
     *
     * @return string
     * @since 1.2.0
     * @since 1.2.4 On Product Bundles v1.5.0, woo_bundles_item_subtotal function is now deprecated. Used item_subtotal function instead.
     */
    public function ecwBundlesItemSubtotal( $subtotal, $item, $itemKey ){


        if( class_exists( 'WC_PB_Cart') ){

            $wcPBCart = WC_PB_Cart::instance();
            return $wcPBCart->item_subtotal( $subtotal, $item, $itemKey );

        }else
            return $subtotal;

    }

    /**
     * Check if the item is bundled parent.
     *
     * @param array $item
     *
     * @return bool
     * @since 1.2.0
     */
    public static function ecwCheckIfBundledParent( $item ){

        if( isset( $item[ 'bundled_items' ] ) && ! isset( $item[ 'bundled_by' ] ) &&
            ! isset( $item[ 'composite_item'] ) )
            return true;
        else
            return false;

    }

    /**
     * Set http $_REQUEST variables to be used before adding bundled items to cart.
     *
     * @param array $item
     * @param int $productID
     * @param int $quantity
     * @param object $product
     *
     * @return bool
     * @since 1.2.0
     * @since 1.2.4 On Product Bundles v1.5.0, they changed it to accept $_POST or $_GET method.
     *              Their Note: We will not rely on $_REQUEST because checkbox names may not exist in $_POST but they may well exist in $_GET, for instance when editing a bundle from the cart.
     */
    public function ecwSetBundledRequestVariables( $item, $productID, $quantity, $product ){

        $bundledItems = $product->get_bundled_items();

        if( ! empty( $item[ 'stamp' ] ) && ! empty( $bundledItems ) ){

            foreach ( $bundledItems as $bundleID => $bundleData ){

                if( array_key_exists( $bundleID, $item[ 'stamp' ] ) ){

                    if( isset( $item[ 'stamp' ][ $bundleID ][ 'optional_selected' ] ) &&
                        $item[ 'stamp' ][ $bundleID ][ 'optional_selected' ] == 'yes' ){
                        $_POST[ 'bundle_selected_optional_' . $bundleID ] = 'yes';
                    }

                    $_POST[ 'bundle_quantity_' . $bundleID ] = (int) $item[ 'stamp' ][ $bundleID ][ 'quantity' ];

                    if( ECW_Helper::get_product_type( $bundleData->product ) == 'variable' ){

                        if( ! empty( $item[ 'stamp' ][ $bundleID ][ 'variation_id' ] ) )
                            $_POST[ 'bundle_variation_id_' . $bundleID ] = (int) $item[ 'stamp' ][ $bundleID ][ 'variation_id' ];

                        if( ! empty( $item[ 'stamp' ][ $bundleID ][ 'attributes' ] ) ){

                            foreach ( $item[ 'stamp' ][ $bundleID ][ 'attributes' ] as $attr => $val ){
                                $_POST[ 'bundle_' . $attr . '_' . $bundleID ] = $val;
                            }
                        }
                    }
                }else{

                    // There's a posibility that the bundled items maybe updated (added or removed items) so we only need to display
                    // those data that are stored in post meta cart object and don't include the new updated items or attributes
                    $_POST[ 'bundle_quantity_' . $bundleID ] = (int) 0;

                }
            }
        }
    }
}
