<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

class ECW_Composite_Products {

    private static $_instance;

    public static function getInstance(){
        if(!self::$_instance instanceof self)
            self::$_instance = new self;
        return self::$_instance;
    }

    /**
     * Changes the tr class of composited items in all templates to allow their styling.
     *
     * @param string $classname
     * @param array $item
     * @param string $itemKey
     *
     * @return string
     * @since 1.2.0
     */
    public function ecwCompositeTableItemClass( $classname, $item, $itemKey ) {

        if ( isset( $item[ 'composite_data' ] ) && ! empty( $item[ 'composite_parent' ] ) )
            return $classname . ' component_table_item';

        elseif ( isset( $item[ 'composite_data' ] ) && ! empty( $item[ 'composite_children' ] ) )
            return $classname . ' component_container_table_item';

        return $classname;
    }

    /**
     * Adds order item title preambles to cart items ( Composite Attribute Descriptions )..
     *
     * @param string $name
     * @param array $item
     * @param string $itemKey
     *
     * @return string
     * @since 1.2.0
     */
    public function ecwCompositeItemName( $name, $item, $itemKey ){

        if( class_exists( 'WC_CP_Display') ){

            $wcCPCart = WC_CP_Display::instance();
            return $wcCPCart->in_cart_component_title( $name, $item, $itemKey );

        }else
            return $name;

    }

    /**
     * Modify the edit screen product listings price of bundled items and container items depending on the bundles's pricing strategy.
     *
     * @param string $price
     * @param array $item
     * @param string $itemKey
     *
     * @return string
     * @since 1.2.0
     */
    public function ecwCompositeItemPrice( $price, $item, $itemKey ){

        if( class_exists( 'WC_CP_Cart') ){

            $wcCPCart = WC_CP_Cart::instance();
            return $wcCPCart->cart_item_price( $price, $item, $itemKey );

        }else
            return $price;

    }

    /**
     * Modify the edit screen product listings subtotal of composite items.
     *
     * @param int $qty
     * @param array $item
     * @param string $itemKey
     *
     * @return string
     * @since 1.2.0
     */
    public function ecwCompositeItemQuantity( $qty, $item, $itemKey ){

        if( class_exists( 'WC_CP_Cart') ){

            $wcCPCart = WC_CP_Cart::instance();
            return $wcCPCart->cart_item_quantity( $qty, $itemKey );

        }else
            return $qty;

    }

    /**
     * Modify the edit screen product listings subtotal of bundled items and container items depending on the bundles's pricing strategy.
     *
     * @param string $subtotal
     * @param array $item
     * @param string $itemKey
     *
     * @return string
     * @since 1.2.0
     */
    public function ecwCompositeItemSubtotal( $subtotal, $item, $itemKey ){

        if( class_exists( 'WC_CP_Cart') ){

            $wcCPCart = WC_CP_Cart::instance();
            return $wcCPCart->item_subtotal( $subtotal, $item, $itemKey );

        }else
            return $subtotal;

    }

    /**
     * Check if the item is a composite parent.
     *
     * @param array $item
     *
     * @return boolean
     * @since 1.2.0
     */
    public static function ecwCheckIfCompositeParent( $item ){

        if( isset( $item[ 'composite_children' ] ) && ! isset( $item[ 'composite_parent' ] ) )
            return true;
        else
            return false;

    }

    /**
     * Set http $_REQUEST variables to be used before adding composite component items to cart.
     *
     * @param array $item
     * @param int $productID
     * @param int $quantity
     * @param object $product
     *
     * @return bool
     * @since 1.2.0
     * @since 1.2.4 On Coposite Products v3.7.0, they changed it to accept $_POST or $_GET method.
     *              Their Note: We will not rely on $_REQUEST because a field name may not exist in $_POST but may well exist in $_GET, for instance when editing a composite from the cart.
     * @since 1.3.0 ECW-125
     */
    public function ecwSetCompositeRequestVariables( $item, $productID, $quantity, $product ){

        $componentsToAdd        = array();
        $componentsToAddQty     = array();
        $componentsVariationID  = array();

        if( ! empty( $item[ 'composite_data' ] ) ){

            foreach ( $item[ 'composite_data' ] as $componentID => $composite ){

                $componentsToAdd[ $componentID ]       = $composite[ 'product_id' ]; // The actual product id
                $componentsToAddQty[ $componentID ]    = (int) ( $composite[ 'quantity' ] != $quantity ) ? $composite[ 'quantity' ] * $quantity : $quantity; // Component product qty

                if( isset( $composite[ 'type' ] ) && $composite[ 'type' ] == 'variable' ){

                    $componentsVariationID[ $componentID ] = ! empty( $composite[ 'variation_id' ] ) ? $composite[ 'variation_id' ] : 0;

                    if( ! empty( $composite[ 'attributes' ] ) && ! empty( $composite[ 'variation_id' ] ) ){

                        $component           = wc_get_product( $composite[ 'product_id' ] );
                        $attributes          = $component->get_variation_attributes();

                        if( ! empty( $attributes ) ){
                            foreach ( $attributes as $variationName => $variation ){

                                $attrVal = '';
                                if( ! empty( $composite[ 'attributes' ][ 'attribute_' . sanitize_title( $variationName ) ] ) )
                                    $attrVal = $composite[ 'attributes' ][ 'attribute_' . sanitize_title( $variationName ) ];

                                $_POST[ 'wccp_attribute_' . sanitize_title( $variationName ) ][ $componentID ] = $attrVal;

                            }
                        }
                    }

                }elseif( isset( $composite[ 'type' ] ) && $composite[ 'type' ] == 'bundle' ){

                    $bundleProduct  = wc_get_product( $composite[ 'product_id' ] );
                    $bundledItems   = $bundleProduct->get_bundled_items();

                    if( ! empty( $composite[ 'stamp' ] ) && ! empty( $bundledItems ) ){

                        foreach ( $bundledItems as $bundleID => $bundleData ){

                            if( array_key_exists( $bundleID, $composite[ 'stamp' ] ) ){

                                if( isset( $composite[ 'stamp' ][ $bundleID ][ 'optional_selected' ] ) &&
                                    $composite[ 'stamp' ][ $bundleID ][ 'optional_selected' ] == 'yes' ){
                                    $_POST[ 'component_' . $componentID . '_bundle_selected_optional_' . $bundleID ] = 'yes';
                                }

                                $_POST[ 'component_' . $componentID . '_bundle_quantity_' . $bundleID ] = (int) $composite[ 'stamp' ][ $bundleID ][ 'quantity' ];

                                if( ECW_Helper::get_product_type( $bundleData->product ) == 'variable' ){

                                    $_POST[ 'component_' . $componentID . '_bundle_variation_id_' . $bundleID ] = (int) $composite[ 'stamp' ][ $bundleID ][ 'variation_id' ];

                                    if( ! empty( $composite[ 'stamp' ][ $bundleID ][ 'attributes' ] ) ){

                                        foreach ( $composite[ 'stamp' ][ $bundleID ][ 'attributes' ] as $attr => $val ){
                                            $_POST[ 'component_' . $componentID . '_bundle_' . $attr . '_' . $bundleID ] = $val;
                                        }
                                    }
                                }
                            }else{

                                // There's a posibility that the bundled items maybe updated (added or removed items) so we only need to display
                                // those data that are stored in post meta cart object and don't include the new updated items or attributes
                                $_POST[ 'component_' . $componentID . '_bundle_quantity_' . $bundleID ] = (int) 0;

                            }
                        }
                    }
                }
            }
        }

        // Pre-fill the request (required to add composite components correctly)
        if ( ! empty( $componentsToAdd ) && ! empty( $componentsToAddQty ) ){

            $_POST[ 'wccp_component_selection' ] = $componentsToAdd;
            $_POST[ 'wccp_component_quantity' ]  = $componentsToAddQty;

            if( ! empty( $componentsVariationID ) )
                $_POST[ 'wccp_variation_id' ] = $componentsVariationID;

        }
    }
}
