<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

class ECW_Cron {

    private static $_instance;

    public static function getInstance(){
        if(!self::$_instance instanceof self)
            self::$_instance = new self;
        return self::$_instance;
    }

    /**
     * Remove expired cart
     *
     * @param int $cartID
     * @param string $cartStatus
     * @param string $cartPostType
     *
     * @since 1.1.0
     */
    public function ecwRemoveExpiredCart( $cartID, $cartStatus, $cartPostType ){

        do_action( 'ecw_before_cart_expired', $cartID, $cartStatus, $cartPostType );

        $cartStatus     = get_post_status( $cartID );
        $cartPostType   = get_post_type( $cartID );

        if( $cartPostType == ECW_CPT_NAME && $cartStatus == 'publish' )
            wp_delete_post( $cartID, true );

        do_action( 'ecw_after_cart_expired', $cartID, $cartStatus, $cartPostType );

    }


    /**
     * Set cron event schedule when the cart will be expired or deleted. Default is 7 days.
     *
     * @param int $cartID
     *
     * @since 1.1.0
     */
    public function ecwSetCartExpiration( $cartID ){
        
        $proceed = apply_filters( 'ecw_proceed_setting_cart_expiration', true, $cartID );
        
        if( ! $proceed )
            return;

        $today          = current_time( 'Y-m-d H:i:s', true );
        $cartStatus     = get_post_status( $cartID );
        $cartPostType   = get_post_type( $cartID );
        $cartAgeLimit   = apply_filters( 'ecw_cart_age_limit', '7 Days' );
        $timeToExecute  = strtotime( '+' . $cartAgeLimit, strtotime( $today ) );
        $args           = array( $cartID, $cartStatus, $cartPostType );

        wp_schedule_single_event( $timeToExecute, ECW_CRON_HOOK, $args );

        // Store cron args into postmeta
        update_post_meta( $cartID, ECW_CART_EXPIRATION_CRON_ARGS, $args );

        do_action( 'ecw_set_cart_expiration', $cartID, $args );

    }

    /**
     * Delete scheduled cart expiration cron event schedule if cart link is deleted.
     *
     * @param int $cartID
     *
     * @since 1.1.0
     * @since 1.2.0 We now get/store the args in postmeta instead of getting it in global options.
     */
    public function ecwDeleteScheduledCartExpirationCron( $cartID ){

        $cartExpArgs = get_post_meta( $cartID, ECW_CART_EXPIRATION_CRON_ARGS, true );
        
        if( ! empty( $cartExpArgs ) && is_int( $cartID ) ){

            $timestamp = wp_next_scheduled( ECW_CRON_HOOK, $cartExpArgs );
            wp_unschedule_event( $timestamp, ECW_CRON_HOOK, $cartExpArgs );

            delete_post_meta( $cartID, ECW_CART_EXPIRATION_CRON_ARGS );

        }

        do_action( 'ecw_delete_scheduled_cart_expiration_cron', $cartID );

    }
}