<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

class ECW_Custom_Meta_Boxes {

    private static $_instance;

    public static function getInstance(){
        if(!self::$_instance instanceof self)
            self::$_instance = new self;
        return self::$_instance;
    }

    /**
     * ECW custom meta boxes
     *
     * @since 1.0.0
     */
    public function ecwShoppingCartMetaBoxes(){

        add_meta_box( 'ecw-shopping-cart-expiry', __( 'Cart Expiry', 'email-cart-for-woocommerce' ), array( self::getInstance(), 'ecwCartExpiryBox' ), ECW_CPT_NAME, 'side', 'default', 'default' );
        add_meta_box( 'ecw-shopping-cart-details-box', __( 'Shopping Cart Details', 'email-cart-for-woocommerce' ), array( self::getInstance(), 'ecwShoppingCartDetails'), ECW_CPT_NAME, 'normal', 'core' );
        add_meta_box( 'ecw-shopping-cart-user-details-box', __( 'User Details', 'email-cart-for-woocommerce' ), array( self::getInstance(), 'ecwShoppingCartUserDetails'), ECW_CPT_NAME, 'normal', 'core' );
        add_meta_box( 'ecw-shopping-cart-product-details-box', __( 'Ordered Items', 'email-cart-for-woocommerce' ), array( self::getInstance(), 'ecwShoppingCartProductDetails'), ECW_CPT_NAME, 'normal', 'core' );
        add_meta_box( 'ecw-shopping-cart-form-info', __( 'Form Info', 'email-cart-for-woocommerce' ), array( self::getInstance(), 'ecwShoppingCartFormInfo'), ECW_CPT_NAME, 'normal', 'core' );
        add_meta_box( 'ecw-shopping-cart-restore-logs', __( 'Cart Restore Logs', 'email-cart-for-woocommerce' ), array( self::getInstance(), 'ecwShoppingCartRestoreLogs'), ECW_CPT_NAME, 'normal', 'core' );

        // We only hide the submitdiv on free plugin
        if( apply_filters( 'ecw_show_upsells', true ) )
            add_meta_box( 'ecw-shopping-cart-upsells-box', __( 'Premium Add-on', 'email-cart-for-woocommerce' ), array( self::getInstance(), 'ecwShoppingCartUpsellsBanner'), ECW_CPT_NAME, 'side', 'core' );

    }

    /**
     * Cart expiry details.
     *
     * @param object $post
     *
     * @since 1.3.0
     */
    public function ecwCartExpiryBox( $post ){

        $ecwCart    = ECW_Shopping_Cart::getInstance();
        $cartExpiry = $ecwCart->ecwGetCartExpirationDate( $post->ID );
        $cartExpiry = '<p class="ecw-expiration-date"><b>' . $cartExpiry . '</b></p>';

        echo apply_filters( 'ecw_cart_expiry_value', $cartExpiry, $post );

        do_action( 'ecw_cart_expiry_box', $post, $cartExpiry );

    }

    /**
     * Shopping cart retore URL.
     *
     * @param object $post
     *
     * @since 1.0.0
     */
    public function ecwShoppingCartDetails( $post ){

        do_action( 'ecw_before_shopping_cart_details_box', $post );

        $cartID = '';
        $cartLink = ' ';
        if( $post->post_type == 'shopping-cart' && $post->post_status == 'publish' ){
            $cartID = $post->ID;
            $cartLink = trailingslashit( wc_get_cart_url() );
            $cartLink = $cartLink . '?cid=' . md5( $cartID );
        }

        $cartLink = apply_filters( 'ecw_cart_details_url', $cartLink, $post );

        echo '<p><label for="shopping_cart_url">' . __( 'Shopping Cart URL', 'email-cart-for-woocommerce' ) . '<a class="copyLink" id="copy' . $cartID . '" data-clipboard-text="' . $cartLink . '" title="' . __( 'Copy to clipboard', 'email-cart-for-woocommerce' ) . '">' . __( 'Copy Link', 'email-cart-for-woocommerce' ) . '</a></label>';
        echo '<input id="shopping_cart_url" type="text" name="shopping_cart_url" value="' . $cartLink . '" readonly="readonly"></p>';

        do_action( 'ecw_after_shopping_cart_details_box', $post );

    }

    /**
     * Insert User information in the new meta box
     *
     * @param object $post
     *
     * @since 1.0.0
     */
    public function ecwShoppingCartUserDetails( $post ){

        $userID = get_post_meta( $post->ID, '_cart_customer_id', true );
        $userID = apply_filters( 'ecw_user_id', $userID, $post );

        if( is_numeric( $userID ) ){

            $user = get_userdata( $userID );
            $user_meta = array_filter( get_user_meta( $userID ) );

        ?>
            <div class='column'>

                <h4><?php _e( 'General Details', 'email-cart-for-woocommerce' ); ?></h4>

                <label><?php _e( 'Customer:', 'email-cart-for-woocommerce' ); ?></label>
                <?php if( !empty( $user->data->user_login ) ): ?>
                    <a href="<?php echo get_edit_user_link( $userID ); ?>">
                        <?php echo $user->data->user_login; ?>
                    </a>
                <?php endif; ?>
                <label><? _e( 'Email:', 'email-cart-for-woocommerce' ); ?></label>
                <?php if( !empty( $user->data->user_email ) ): ?>
                    <a href="mailto:<?php echo $user->data->user_email; ?>">
                        <?php echo $user->data->user_email; ?>
                    </a>
                <?php endif; ?>

                <label><?php _e( 'Role:', 'email-cart-for-woocommerce' ); ?></label>
                <?php
                    foreach ( $user->roles as $key => $value ) {
                        echo ucwords( str_replace( '_', ' ', $value ) ).'<br>';
                    }
                ?>

            </div>

            <div class='column'>

                <h4><?php _e( 'Billing Details', 'email-cart-for-woocommerce' ); ?></h4>

                <label><?php _e( 'Address:', 'email-cart-for-woocommerce' ); ?></label>
                <?php echo isset( $user_meta[ 'billing_first_name' ][ 0 ] ) ? $user_meta[ 'billing_first_name' ][ 0 ]: ''; ?>
                <?php echo isset( $user_meta[ 'billing_last_name' ][ 0 ] ) ? $user_meta[ 'billing_last_name' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'billing_company' ][ 0 ] ) ? '<br>'.$user_meta[ 'billing_company' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'billing_address_1' ][ 0 ] ) ? '<br>'.$user_meta[ 'billing_address_1' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'billing_address_2' ][ 0 ] ) ? '<br>'.$user_meta[ 'billing_address_2' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'billing_city' ][ 0 ] ) ? '<br>'.$user_meta[ 'billing_city' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'billing_postcode' ][ 0 ] ) ? '<br>'.$user_meta[ 'billing_postcode' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'billing_country' ][ 0 ] ) ? '<br>'.$user_meta[ 'billing_country' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'billing_state' ][ 0 ] ) ? '<br>'.$user_meta[ 'billing_state' ][ 0 ] : ''; ?>

                <label><?php _e( 'Email:', 'email-cart-for-woocommerce' ); ?></label>
                <?php if( !empty( $user_meta[ 'billing_email' ][ 0 ] ) ): ?>
                    <a href="mailto:<?php echo $user_meta[ 'billing_email' ][ 0 ]; ?>">
                        <?php echo $user_meta[ 'billing_email' ][ 0 ]; ?>
                    </a>
                <?php endif; ?>

                <label><?php _e( 'Phone:', 'email-cart-for-woocommerce' ); ?></label>
                <?php echo isset( $user_meta[ 'billing_phone' ][ 0 ] ) ? $user_meta[ 'billing_phone' ][ 0 ] : ''; ?>

            </div>

            <div class='column'>
                <h4><?php _e( 'Shipping Details', 'email-cart-for-woocommerce' ); ?></h4>

                <label><?php _e( 'Address:', 'email-cart-for-woocommerce' ); ?></label>
                <?php echo isset( $user_meta[ 'shipping_first_name' ][ 0 ] ) ? $user_meta[ 'shipping_first_name' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'shipping_last_name' ][ 0 ] ) ? $user_meta[ 'shipping_last_name' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'shipping_company' ][ 0 ] ) ? '<br>'.$user_meta[ 'shipping_company' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'shipping_address_1' ][ 0 ] ) ? '<br>'.$user_meta[ 'shipping_address_1' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'shipping_address_2' ][ 0 ] ) ? '<br>'.$user_meta[ 'shipping_address_2' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'shipping_city' ][ 0 ] ) ? '<br>'.$user_meta[ 'shipping_city' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'shipping_postcode' ][ 0 ] ) ? '<br>'.$user_meta[ 'shipping_postcode' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'shipping_country' ][ 0 ] ) ? '<br>'.$user_meta[ 'shipping_country' ][ 0 ] : ''; ?>
                <?php echo isset( $user_meta[ 'shipping_state' ][ 0 ] ) ? '<br>'.$user_meta[ 'shipping_state' ][ 0 ] : ''; ?>

            </div>
        <?php
        }else{
            _e( 'Unregistered', 'email-cart-for-woocommerce' );
        }

        do_action( 'ecw_shopping_cart_user_details_box', $post );
    }

    /**
    * Remove Yoast SEO meta box and other meta box that is not needed
    *
    * @since 1.0.0
    */
    public function ecwRemoveMetaBoxes(){

        global $post;

        // Hide shopping cart metaboxes in the edit screen
        if( isset( $post ) && isset( $post->post_type ) && $post->post_type === 'shopping-cart' ){

            add_filter( 'wpseo_use_page_analysis', '__return_false' );

            // We only hide the submitdiv on free plugin
            if( apply_filters( 'ecw_hide_submitdiv', true ) )
                remove_meta_box( 'submitdiv', ECW_CPT_NAME, 'side' );

            remove_meta_box( 'wpseo_meta', ECW_CPT_NAME, 'normal' );
            remove_meta_box( 'trackbacksdiv', ECW_CPT_NAME, 'normal' ); // Trackbacks meta box
            remove_meta_box( 'postcustom', ECW_CPT_NAME, 'normal' ); // Custom fields meta box
            remove_meta_box( 'commentsdiv', ECW_CPT_NAME, 'normal' ); // Comments meta box
            remove_meta_box( 'slugdiv', ECW_CPT_NAME, 'normal' ); // Slug meta box
            remove_meta_box( 'authordiv', ECW_CPT_NAME, 'normal' ); // Author meta box
            remove_meta_box( 'revisionsdiv', ECW_CPT_NAME, 'normal' ); // Revisions meta box
            remove_meta_box( 'formatdiv', ECW_CPT_NAME, 'normal' ); // Post format meta box
            remove_meta_box( 'commentstatusdiv', ECW_CPT_NAME, 'normal' ); // Comment status meta box
            remove_meta_box( 'categorydiv', ECW_CPT_NAME, 'side' ); // Category meta box
            remove_meta_box( 'tagsdiv-post_tag', ECW_CPT_NAME, 'side' ); // Post tags meta box
            remove_meta_box( 'pageparentdiv', ECW_CPT_NAME, 'side' ); // Page attributes meta box
            remove_meta_box( 'file_gallery', ECW_CPT_NAME, 'normal'); // File Gallery Plugin
            remove_meta_box( 'wpseo_meta', ECW_CPT_NAME, 'normal'); // YOAST Seo
            remove_meta_box( 'gdsr-meta-box-mur', ECW_CPT_NAME, 'advanced');  // GD Star Rating
            remove_meta_box( 'gdsr-meta-box', ECW_CPT_NAME, 'side' ); // GD Star Rating
            remove_meta_box( 'yourlsdiv', ECW_CPT_NAME, 'side'); // YOURLS

        }

    }

    /**
     * Insert User information in the new meta box
     *
     * @param object $post
     *
     * @since 1.0.0
     */
    public function ecwShoppingCartProductDetails( $post ){

        $ecwShoppingCart    = ECW_Shopping_Cart::getInstance();
        $items              = $ecwShoppingCart->ecwSetCartItems( $post->ID );

        do_action( 'ecw_before_product_details', $post ); ?>

        <div class="ecw_product_details_wrapper">
            <table cellpadding="0" cellspacing="0" class="ecw_product_details" style="width:100%;">
                <thead>
                    <tr>
                        <th class="item" colspan="2"><?php _e( 'PRODUCT', 'email-cart-for-woocommerce' ); ?></th>
                        <th class="item_cost"><?php _e( 'PRICE', 'email-cart-for-woocommerce' ); ?></th>
                        <th class="quantity"><?php _e( 'QUANTITY', 'email-cart-for-woocommerce' ); ?></th>
                        <th class="line_cost"><?php _e( 'TOTAL', 'email-cart-for-woocommerce' ); ?></th>
                    </tr>
                </thead>
                <tbody id="product_line_items"><?php

                    if( ! empty( $items ) ){

                        foreach ( $items->cart_contents as $itemID => $item ){
                            $productID = !empty( $item[ 'variation_id' ] ) ? $item[ 'variation_id' ] : $item[ 'product_id' ];
                            $product = wc_get_product( $productID );
                            $variation = array();
                            $cartItemData = array(); ?>

                            <tr class="<?php echo esc_attr( apply_filters( 'ecw_cart_item_class', 'cart_item', $item, $itemID ) ); ?>" data-product-id="<?php echo $productID; ?>" data-product-variation-id="<?php echo $item[ 'variation_id' ]; ?>">
                                <td class="thumb">
                                    <a href="<?php echo esc_url( admin_url( 'post.php?post=' . absint( $item[ 'product_id' ] ) . '&action=edit' ) ); ?>">
                                    <?php echo $product->get_image( array( 100, 100 ) ); ?>
                                    </a>
                                </td>
                                <td class="product-name"><?php
                                    $productSKU = $product->get_sku();
                                    $productSKU = ! empty( $productSKU ) ? ' - ' . $productSKU : '';
                                    echo apply_filters( 'ecw_cart_item_name', sprintf( '<a href="%s">%s</a>%s', esc_url( admin_url( 'post.php?post=' . absint( $item[ 'product_id' ] ) . '&action=edit' ) ), $product->get_title(), $productSKU ), $item, $itemID );

                                    // Meta data
                                    echo WC()->cart->get_item_data( $item ); ?>

                                </td>
                                <td class="item_cost">
                                    <?php echo apply_filters( 'ecw_cart_item_price', WC()->cart->get_product_price( $product ), $item, $itemID ); ?>
                                </td>
                                <td class="quantity"><?php
                                    $qty = isset( $item[ 'quantity' ] ) ? esc_html( $item[ 'quantity' ] ) : '';
                                    echo apply_filters( 'ecw_cart_item_quantity', $qty, $item, $itemID ); ?>
                                </td>
                                <td class="line_cost">
                                    <?php echo apply_filters( 'ecw_cart_item_subtotal', WC()->cart->get_product_subtotal( $product, $qty ), $item, $itemID ); ?>
                                </td>
                            </tr><?php
                        }
                    }else{ ?>
                        <tr>
                            <td colspan="5"><?php _e( 'No products found.', 'email-cart-for-woocommerce' ); ?></td>
                        </tr><?php
                    } ?>
                </tbody>
            </table>
        </div>

        <div class="ecw-product-data-row ecwp-cart-totals">
            <table class="table totals" cellspacing="0">
                <tr class="cart-subtotal">
                    <th><?php _e( 'SUBTOTAL', 'email-cart-for-woocommerce' ) ?></th>
                    <td><?php wc_cart_totals_subtotal_html(); ?></td>
                </tr>
                <?php foreach ( $items->get_coupons() as $code => $coupon ) : ?>
                    <tr class="cart-discount coupon-<?php echo esc_attr( sanitize_title( $code ) ); ?>">
                        <th><?php wc_cart_totals_coupon_label( $coupon ); ?></th>
                        <td><?php wc_cart_totals_coupon_html( $coupon ); ?></td>
                    </tr>
                <?php endforeach; ?>

                <tr class="order-total">
                    <th><?php _e( 'TOTAL', 'email-cart-for-woocommerce' ); ?></th>
                    <td><?php wc_cart_totals_order_total_html(); ?></td>
                </tr>
            </table>
        </div><?php

        do_action( 'ecw_after_product_details', $post );

    }

    /**
     * Insert User information in the new meta box
     *
     * @param object $post
     *
     * @since 1.2.0
     */
    public function ecwShoppingCartUpsellsBanner( $post ){ ?>

        <style type="text/css">
            div#ecw-shopping-cart-upsells-box div.inside{
                padding: 0px;
                margin: 0px;
                overflow: hidden;
            }
            div#ecw-shopping-cart-upsells-box div.inside a,
            div#ecw-shopping-cart-upsells-box div.inside img{
                float: left;
            }
        </style>
        <a target="_blank" href="https://marketingsuiteplugin.com/product/email-cart/?utm_source=ECW&utm_medium=Settings%20Banner&utm_campaign=ECW">
            <img style="outline: none;" src="<?php echo ECW_IMAGES_URL . 'sidebar-upsells.png'; ?>" alt="<?php _e( 'Email Cart for WooCommerce Premium' , 'email-cart-for-woocommerce' ); ?>"/>
        </a><?php

    }

    /**
     * Display cart restore logs
     *
     * @param object $post
     *
     * @since 1.4.0
     */
    public function ecwShoppingCartRestoreLogs( $post ) {

        $restore_logs = get_post_meta( $post->ID , 'ecw_cart_restore_logs' , true );

        if ( empty( $restore_logs ) ) {

            echo __( 'No cart restores logged yet.' , 'email-cart-for-woocommerce' );
            return;

        }

        ?>

        <table class="table restore-logs-table wp-list-table widefat fixed striped">
            <thead>
                <tr>
                    <th><?php _e( 'Customer' , 'email-cart-for-woocommerce' ); ?></th>
                    <th><?php _e( 'Email' , 'email-cart-for-woocommerce' ); ?></th>
                    <th><?php _e( 'Role' , 'email-cart-for-woocommerce' ); ?></th>
                    <th><?php _e( 'Order' , 'email-cart-for-woocommerce' ); ?></th>
                    <th><?php _e( 'Date' , 'email-cart-for-woocommerce' ); ?></th>
                </tr>
            </thead>
            <tbody>
                <?php foreach ( $restore_logs as $key => $log ) : ?>
                    <tr>
                        <td>
                            <?php if ( $log['userID'] > 0 ) : ?>
                                <a href="<?php echo get_edit_user_link( $log['userID'] ); ?>">
                                    <?php echo esc_attr( $log['customer'] ); ?>
                                </a>
                            <?php else : ?>
                                <?php echo esc_attr( $log['customer'] ); ?>
                            <?php endif; ?>
                        </td>
                        <td><?php echo esc_attr( $log['email'] ); ?></td>
                        <td><?php echo esc_attr( $log['role'] ); ?></td>
                        <td>
                            <a href="<?php echo get_edit_post_link( $log['orderID'] ); ?>">
                                <?php echo esc_attr( $log['orderID'] ); ?>
                            </a>
                            </td>
                        <td><?php echo esc_attr( $log['Date'] ); ?></td>
                    </tr>
                <?php endforeach; ?>
            </tbody>
        </table>
        <?php
    }

    /**
     * Display cart form information
     *
     * @param object $post
     *
     * @since 1.4.0
     */
    public function ecwShoppingCartFormInfo( $post ) {

        $formInfo = get_post_meta( $post->ID , 'ecw_cart_form_info' , true );

        if ( empty( $formInfo ) || ! is_array( $formInfo ) ) {
            echo __( 'No information available.' , 'email-cart-for-woocommerce' );
            return;
        }

        $labels = array(
            'sendersName'       => __( "Sender's Name:" , 'email-cart-for-woocommerce' ),
            'recipientName'     => __( "Recipient's Name:" , 'email-cart-for-woocommerce' ),
            'recipientEmail'    => __( "Recipient's Email:" , 'email-cart-for-woocommerce' ),
            'subject'           => __( "Email Subject:" , 'email-cart-for-woocommerce' ),
            'message'           => __( "Email Message:" , 'email-cart-for-woocommerce' )
        );

        if ( ! is_plugin_active( 'email-cart-for-woocommerce-premium/email-cart-for-woocommerce-premium.bootstrap.php' ) )
            $formInfo['message'] = $formInfo['subject'] = __( '<em>Premium only</em>' , 'email-cart-for-woocommerce' );

        ?>
            <table class="table form-info-table wp-list-table widefat">
                <?php foreach ( $formInfo as $key => $info ) : ?>
                    <tr>
                        <th><strong><?php echo $labels[ $key ]; ?></strong></th>
                        <td><?php echo $info; ?></td>
                    </tr>
                <?php endforeach; ?>
            </table>
        <?php

    }
}
