<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

class ECW_Emails {

    private static $_instance;

    public static function getInstance(){
        if(!self::$_instance instanceof self)
            self::$_instance = new self;
        return self::$_instance;
    }

    /**
     * Adds email cart button in woocommerce cart page
     *
     * @since 1.0.0
     * @since 1.4.0 added checks for restricted user roles
     */
    public function ecwAddEmailCartButtonInCartPage(){

        do_action( 'ecw_before_email_cart_button' );

        $roles = is_user_logged_in() ? wp_get_current_user()->roles : array( 'guest' );

        $is_user_restricted = array_intersect( $roles , get_option( 'ecw_email_exclude_user_roles', array() ) );

        if ( empty( $is_user_restricted ) )
            echo '<button type="button" class="button email_cart" id="email_cart" name="email_cart"><span class="dashicons dashicons-email-alt"></span>' . apply_filters( 'ecw_email_cart_button', __( 'Email Cart', 'email-cart-for-woocommerce' ) ) . '</button>';

        do_action( 'ecw_after_email_cart_button' );

    }

    /**
     * Sets the Email Cart Form Modal Box HTML
     *
     * @since 1.0.0
     * @since 1.4.0 added checks for restricted user roles
     * @since 1.4.5 added wp_nonce_field() to avoid any unwanted request when processing email send
     */
    public function ecwSetEmailCartFormModalBoxHTML(){

        if( is_cart() ){

            do_action( 'ecw_before_email_cart_form_modal' );

            $roles = is_user_logged_in() ? wp_get_current_user()->roles : array( 'guest' );

            $is_user_restricted = array_intersect( $roles , get_option( 'ecw_email_exclude_user_roles', array() ) );

            if ( empty( $is_user_restricted ) )
                echo apply_filters( 'ecw_email_cart_form_modal', '<div id="email-cart-form" title="Warning" style="display:none">
                        <span class="ui-state-default"><span class="ui-icon ui-icon-info" style="float:left; margin:0 7px 0 0;"></span></span>
                        <div class="form">' .
                            wp_nonce_field( 'email-cart-send' , 'email-cart-form-nonce-field' ) .
                            '<p>' . __( 'Enter the recipient\'s name and email below and they will be emailed a link to this pre-filled shopping cart.', 'email-cart-for-woocommerce' ) . '</p>
                            <p><input type="text" placeholder="' . __( 'Your Name:', 'email-cart-for-woocommerce' ) . '" id="sender_name"></p>
                            <p><input type="text" placeholder="' . __( 'Recipient Name:', 'email-cart-for-woocommerce' ) . '" id="recipient_name"></p>
                            <p><input type="text" placeholder="' . __( 'Recipient Email Address:', 'email-cart-for-woocommerce' ) . '" id="recipient_email"></p>
                        </div>
                        <div class="process" style="display:none">
                            <img src="' . ECW_IMAGES_URL . 'spinner-2x.gif">
                        </div>
                        <div class="success" style="display:none">
                            <img src="' . ECW_IMAGES_URL . 'img-success.png">
                            <p>' . __( 'Your shopping cart has been sent.', 'email-cart-for-woocommerce' ) . '</p>
                        </div>
                        <div class="error" style="display:none">
                            <img src="' . ECW_IMAGES_URL . 'img-error.png">
                            <p>' . __( 'Sending cart failed. Please try again.', 'email-cart-for-woocommerce' ) . '</p>
                        </div>
                    </div>' );

            do_action( 'ecw_after_email_cart_form_modal' );

        }

    }

    /**
     * Set/Filter wp_mail "From" Header
     *
     * @return string
     * @since 1.0.0
     */
    public function ecwWPMailFrom() {

        $fromEmail = trim( get_option( 'woocommerce_email_from_address' ) );

        if ( ! $fromEmail )
            $fromEmail = get_option( 'admin_email' );

        return apply_filters( 'ecw_email_from_email' , $fromEmail );

    }

    /**
     * Set/Filter wp_mail 'From Name' Header
     *
     * @return string
     * @since 1.0.0
     */
    public function ecwWPMailFromName( $sendersName ) {

        $wcFromName = trim( get_option( 'woocommerce_email_from_name' ) );

        if ( empty( $sendersName ) )
            $sendersName = $wcFromName;

        return apply_filters( 'ecw_email_from_name' , $sendersName );

    }

    /**
     * Construct email headers.
     *
     * @param $fromName
     * @param $fromEmail
     *
     * @return array
     * @since 1.0.0
     */
    public function ecwConstructEmailHeader( $fromName, $fromEmail ) {

        $headers[] = 'From: ' . $fromName . ' < ' . $fromEmail . ' > ';

        $headers[] = apply_filters( 'ecw_email_content_type', 'Content-Type: text/html;' );

        $headers[] = apply_filters( 'ecw_email_charset', 'charset=UTF-8' );

        return apply_filters( 'ecw_email_header', $headers );

    }
}
