<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

class ECW_Guided_Tour {
    
    private static $_instance;
    private $urls;
    private $screens;

    public static function getInstance(){
        if(!self::$_instance instanceof self)
            self::$_instance = new self;
        return self::$_instance;
    }

    /**
     * ECW_Guided_Tour constructor.
     *
     * @since 1.3.0
     */
    private function __construct() {

        $this->urls = apply_filters( 'ecw_guided_tour_pages' , array(
            'plugin-listing'    => admin_url( 'plugins.php' ),
            'orders-listing'    => admin_url( 'edit.php?post_type=shop_order' ),
            'ecw-listing'       => admin_url( 'edit.php?post_type=shopping-cart' )
        ));

        $this->screens = apply_filters( 'acr_guided_tours' , array(
            'plugins' => array(
                'elem'  => '#menu-plugins .menu-top',
                'html'  => __( '<h3>Welcome to Email Cart for WooCommerce!</h3>
                                <p>Would you like to go on a guided tour of the plugin? Takes less than 30 seconds.</p>' , 'email-cart-for-woocommerce' ),
                'prev'  => null,
                'next'  => $this->urls[ 'orders-listing' ],
                'edge'  => 'left',
                'align' => 'left'
            ),
            'edit-shop_order' => array(
                'elem'  => '#toplevel_page_woocommerce ul li a.current',
                'html'  => __( '<h3>Email Cart for WooCommerce.</h3>
                                <p>About ECW.</p>' ),
                'prev'  => $this->urls[ 'plugin-listing' ],
                'next'  => $this->urls[ 'ecw-listing' ],
                'edge'  => 'left',
                'align' => 'left'
            ),
            'edit-shopping-cart' => array(
                'elem'  => '#toplevel_page_woocommerce ul li a.current',
                'html'  => __( '<h3>ECW Listings.</h3>
                                <p>Wordings.</p>', 'email-cart-for-woocommerce' ),
                'prev'  => $this->urls[ 'orders-listing' ],
                'next'  => null,
                'edge'  => 'left',
                'align' => 'left'
            )
        ));

    }

    /**
     * Get current screen.
     * 
     * @since 1.3.0
     */
    public function ecwGetCurrentScreen() {
        
        $screen = get_current_screen();

        if ( ! empty( $this->screens[ $screen->id ] ) ){

            return $this->screens[ $screen->id ];

        }

        return false;

    }

    /**
     * Get screens with registered guide.
     * 
     * @since 1.3.0
     */
    public function ecwGetScreens() {

        return $this->screens;

    }

    /**
     * Close initial guided tour.
     * 
     * @since 1.3.0
     */
    public function ecwCloseGuidedTour() {

        if ( is_admin() && defined( 'DOING_AJAX' ) && DOING_AJAX ) {

            if ( !check_ajax_referer( 'ecw-close-guided-tour', 'nonce', false ) )
                wp_die( __( 'Security Check Failed', 'email-cart-for-woocommerce' ) );

            update_option( 'ecw_guided_tour_status', 'close' );

            wp_send_json_success();

        } else
            wp_die( __( 'Invalid AJAX Call', 'email-cart-for-woocommerce' ) );
        
    }
}