<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

final class ECW_Helper {

    /**
     * Get data about the current woocommerce installation.
     *
     * @since 1.3.1
     * @access public
     * @return array Array of data about the current woocommerce installation.
     */
    public static function get_woocommerce_data() {

        if ( ! function_exists( 'get_plugin_data' ) )
            require_once( ABSPATH . '/wp-admin/includes/plugin.php' );

        return get_plugin_data( WP_PLUGIN_DIR . '/woocommerce/woocommerce.php' );

    }

    /**
     * Get order properties based on the key. WC 3.0
     *
     * @since 1.3.1
     * @access public
     *
     * @param WC_Order $order  order object
     * @param string   $key    order property
     * @return string   order property
     */
    public static function get_order_data( $order , $key ) {

        if ( is_a( $order , 'WC_Order' ) ) {

            $woocommerce_data = self::get_woocommerce_data();

            if ( version_compare( $woocommerce_data[ 'Version' ] , '3.0.0' , '>=' ) ) {

                switch ( $key ) {

                    case 'modified_date' :
                        return $order->get_date_modified() ? gmdate( 'Y-m-d H:i:s', $order->get_date_modified()->getOffsetTimestamp() ) : '';
                        break;

                    case 'completed_date' :
                        return $order->get_date_completed() ? gmdate( 'Y-m-d H:i:s', $order->get_date_completed()->getOffsetTimestamp() ) : '';
                        break;

                    case 'order_date' :
                        return $order->get_date_created() ? gmdate( 'Y-m-d H:i:s', $order->get_date_created()->getOffsetTimestamp() ) : '';
                        break;

                    case 'order_total' :
                        return $order->get_total();
                        break;

                    case 'customer_user' :
                        return $order->get_customer_id();
                        break;

                    default:
                        $key = 'get_' . $key;
                        return $order->$key();
                        break;
                }

            }
            else
                return $order->$key;

        } else {

            error_log( 'ECW Error : get_order_data helper functions expect parameter $order of type WC_Order.' );
            return 0;

        }
    }

    /**
	 * Get product type. WC 3.0
	 *
	 * @since 1.4.3
     *
	 * @param WC_Product $product Product type.
	 * @return string Product type.
	 */
    public static function get_product_type( $product ) {

        if ( is_a( $product , 'WC_Product' ) ) {

            $woocommerce_data = self::get_woocommerce_data();

            if ( version_compare( $woocommerce_data[ 'Version' ] , '3.0.0' , '>=' ) )
                return $product->get_type();
            else
                return $product->product_type;

        } else {

            error_log( 'ECW Error : get_product_type function expect parameter $product of type WC_Product.' );
            return 0;

        }
    }

}
