<?php
/**
 * Email Cart for WooCommerce Settings
 *
 * @author      Rymera Web Co
 * @version     1.3.0
 */
if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! class_exists( 'ECW_Settings' ) ) {

	class ECW_Settings extends WC_Settings_Page {

	    public function __construct() {

            $this->id    = 'ecw_settings';
            $this->label = __( 'Email Cart', 'email-cart-for-woocommerce' );

            // Show upsell graphics
            if( apply_filters( 'ecw_show_upsells', true ) )
                add_action( 'woocommerce_admin_field_ecw_upsell' , array( $this , 'ecwRenderUpsellGraphics' ) );

            add_action( 'woocommerce_settings_save_' . $this->id, array( $this, 'ecwSave' ), 99 );
            add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_page' ), 30 );
            add_action( 'woocommerce_settings_' . $this->id, array( $this, 'ecwOutput' ), 10 );
            add_action( 'woocommerce_sections_' . $this->id, array( $this, 'output_sections' ) );
            add_filter( 'woocommerce_get_sections_' . $this->id, array( $this, 'ecwGetSections' ) );
            add_action( 'woocommerce_admin_field_ecw_help_resources', array( $this, 'ecwRenderHelpResources' ) );
            add_action( 'ecw_css_tweaks', array( $this, 'ecwStyleTweaks' ) );
	    }

        /**
         * Style tweaks.
         * Ex. hide button in general tab
         *
         * @param string|null $section
         *
         * @since 1.3.0
         */
        public function ecwStyleTweaks( $section ){

            if( $section == '' && ! is_plugin_active( 'email-cart-for-woocommerce-premium/email-cart-for-woocommerce-premium.bootstrap.php' ) ){ ?>
                <style type="text/css">
                    p.submit .woocommerce-save-button{
                        /* display: none; */
                    }
                </style><?php
            }

        }

        /**
         * Save settings.
         *
         * @since 1.3.0
         */
        public function ecwSave() {

            global $current_section;

            $settings = $this->ecwGetSettings( $current_section );

            WC_Admin_Settings::save_fields( $settings );

            do_action( $this->id . '_save', $current_section, $settings );

        }

        /**
         * Output the settings.
         *
         * @since 1.3.0
         */
        public function ecwOutput() {

            global $current_section;

            do_action( 'ecw_css_tweaks', $current_section );

            $settings = $this->ecwGetSettings( $current_section );

            WC_Admin_Settings::output_fields( $settings );

            do_action( $this->id . '_output', $current_section, $settings );

        }

        /**
         * Get sections.
         *
         * @return array
         * @since 1.3.0
         */
        public function ecwGetSections() {

            $sections = array(
                ''      =>  __( 'General', 'email-cart-for-woocommerce' ),
                'help' 	=>  __( 'Help', 'email-cart-for-woocommerce' )
            );

            return apply_filters( 'ecw_get_sections_' . $this->id, $sections );

        }

        /**
         * Get settings array.
         *
         * @param string|null $section
         *
         * @return mixed
         * @since 1.3.0
         */
        public function ecwGetSettings( $section = '' ) {

            $settings = $this->ecwGetSectionSettings( $section );

            return apply_filters( 'ecw_get_settings_' . $this->id, $settings, $section );

        }

        /**
         * Get section settings.
         *
         * @param string|null $section
         *
         * @return array
         * @since 1.3.0
		 * @since 1.4.0	added "excluded user role" setting
         */
        private function ecwGetSectionSettings( $section ){

            $settings = array();

        	switch ( $section ) {

                case '':

				global $wp_roles;
				$roles = array();

				$allRoles = $wp_roles->roles;
				foreach ( $allRoles as $key => $val ) {
					$roles[ $key ] = $val[ 'name' ];
				}

				// Include Guest
				$roles[ 'guest' ] = 'Guest';

				$roles = apply_filters( 'acrp_exclude_user_roles_list', $roles );

                    $settings = array(
                            array(
                                'title' => __( 'General Options', 'email-cart-for-woocommerce' ),
                                'type'  => 'title',
                                'desc'  => '',
                                'id'    => 'ecw_general_main_title'
                            ),
                            array(
                                'type'  => 'ecw_upsell',
                                'id'    => 'ecw_upsell',
                            ),
							array(
		                        'title'     => __( 'Exclude User Roles', 'advanced-cart-recovery-premium' ),
		                        'type'      => 'multiselect',
		                        'desc'      => '',
		                        'desc_tip'  => __( 'Excludes the nominated user roles.', 'email-cart-for-woocommerce' ),
		                        'id'        => 'ecw_email_exclude_user_roles',
								'class'		=> 'wc-enhanced-select',
		                        'css'       => 'width:400px;',
		                        'options'   =>  $roles
		                    ),
    		                array(
    		                    'type'  => 'sectionend',
    		                    'id'    => 'ecw_cron_sectionend'
    		                )
                        );

                    return apply_filters( $this->id . '_general', $settings );

                    break;

        		case 'help':

		            $settings = array(

    		                array(
    		                    'title' => __( 'Help Options', 'email-cart-for-woocommerce' ),
    		                    'type'  => 'title',
    		                    'desc'  => '',
    		                    'id'    => 'ecw_help_main_title'
    		                ),
    		                array(
    		                    'title' => '',
    		                    'type'  => 'ecw_help_resources',
    		                    'desc'  => '',
    		                    'id'    => 'ecw_help_resources'
    		                ),
                            array(
                                'title' => __( 'Clean up plugin options on un-installation', 'email-cart-for-woocommerce' ),
                                'type'  => 'checkbox',
                                'desc'  => __( 'If checked, removes all plugin options when this plugin is uninstalled. Note: Also affect premium version.', 'email-cart-for-woocommerce' ),
                                'id'    => 'ecw_clean_plugin_options',
                            ),
    		                array(
    		                    'type'  => 'sectionend',
    		                    'id'    => 'ecw_cron_sectionend'
    		                )
		              );

                    return apply_filters( $this->id . '_' . $section, $settings );

        			break;

                default:

                    return apply_filters(  $this->id . '_get_section', $section  );

        			break;

        	}

        }

        /**
         * Render knowledge base link
         *
         * @param array $data
         *
         * @since 1.3.0
         */
        public function ecwRenderHelpResources( $data ){

            do_action( 'ecw_render_help_before', $data ); ?>

            <tr valign="top">
                <th scope="row" class="titledesc">
                    <label for=""><?php _e( 'Knowledge Base', 'email-cart-for-woocommerce' ); ?></label>
                </th>
                <td class="forminp forminp-<?php echo sanitize_title( $data[ 'type' ] ); ?>">
                    <?php echo sprintf( __( 'Looking for documentation? Please see our growing <a href="%1$s" target="_blank">Knowledge Base</a>', 'email-cart-for-woocommerce' ), 'https://marketingsuiteplugin.com/knowledge-base/email-cart/?utm_source=ECW&utm_medium=Settings%20Help&utm_campaign=ECW' ); ?>
                </td>
            </tr><?php

            do_action( 'ecw_render_help_after', $data );

        }

        /**
         * Render upsell graphic.
         *
         * @param array $data
         *
         * @since 1.3.0
         */
        public function ecwRenderUpsellGraphics( $data ){ ?>

            <tr valign="top">
                <th scope="row" class="titledesc" colspan="2">
                    <a target="_blank" href="https://marketingsuiteplugin.com/product/email-cart/?utm_source=ECW&utm_medium=Settings%20Banner&utm_campaign=ECW">
                        <img style="outline: none;" src="<?php echo ECW_IMAGES_URL . 'general-upsells.png'; ?>" alt="<?php _e( 'Email Cart for WooCommerce Premium' , 'email-cart-for-woocommerce' ); ?>"/>
                    </a>
                </th>
            </tr><?php

        }
	}
}

return new ECW_Settings();
