<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

class ECW_Shopping_Cart {

    private static $_instance;

    public static function getInstance(){
        if(!self::$_instance instanceof self)
            self::$_instance = new self;
        return self::$_instance;
    }

    /**
     * Instantiate WC session, cart and customer
     *
     * @since 1.0.0
     */
    public function ecwWooCommerceInit(){

        include_once( WC()->plugin_path() . '/includes/abstracts/abstract-wc-session.php' );
        include_once( WC()->plugin_path() . '/includes/wc-cart-functions.php' );
        include_once( WC()->plugin_path() . '/includes/wc-notice-functions.php' );

        WC()->session  = new WC_Session_Handler();
        WC()->cart     = new WC_Cart();
        WC()->customer = new WC_Customer();

    }

    /**
     * Used to register new Shopping Cart CPT
     *
     * @since 1.0.0
     */
    public function ecwRegisterShoppingCartCPT(){

        $labels = array(
            'name'                  => __( 'Email Cart Links', 'email-cart-for-woocommerce' ),
            'singular_name'         => __( 'Email Cart Link', 'email-cart-for-woocommerce' ),
            'menu_name'             => __( 'Email Cart Links', 'email-cart-for-woocommerce' ),
            'name_admin_bar'        => __( 'Email Cart Links', 'email-cart-for-woocommerce' ),
            'parent_item_colon'     => __( 'Parent Cart:', 'email-cart-for-woocommerce' ),
            'all_items'             => __( 'Email Cart Links', 'email-cart-for-woocommerce' ),
            'add_new_item'          => __( 'Add New Cart', 'email-cart-for-woocommerce' ),
            'add_new'               => __( 'Add New', 'email-cart-for-woocommerce' ),
            'new_item'              => __( 'New Cart', 'email-cart-for-woocommerce' ),
            'edit_item'             => __( 'Edit Cart', 'email-cart-for-woocommerce' ),
            'update_item'           => __( 'Update Cart', 'email-cart-for-woocommerce' ),
            'view_item'             => __( 'View Cart', 'email-cart-for-woocommerce' ),
            'search_items'          => __( 'Search Email Cart Link', 'email-cart-for-woocommerce' ),
            'not_found'             => __( 'Not found', 'email-cart-for-woocommerce' ),
            'not_found_in_trash'    => __( 'Not found in Trash', 'email-cart-for-woocommerce' ),
        );

        $labels = apply_filters( 'ecw_shopping_cart_cpt_labels', $labels );

        $args = array(
            'label'                 => __( 'email-cart-links', 'email-cart-for-woocommerce' ),
            'description'           => __( 'Email Cart Links', 'email-cart-for-woocommerce' ),
            'labels'                => $labels,
            'supports'              => array( 'title' ),
            'hierarchical'          => false,
            'public'                => false,
            'show_ui'               => true,
            'show_in_menu'          => 'woocommerce',
            'menu_position'         => 5,
            'show_in_admin_bar'     => true,
            'show_in_nav_menus'     => true,
            'can_export'            => true,
            'has_archive'           => true,
            'exclude_from_search'   => false,
            'publicly_queryable'    => false,
            'capability_type'       => 'post',
            'capabilities'          => array(
                                        'create_posts' => false, // Removes support for the "Add New" function
            ),
            'map_meta_cap'          => true,
            'rewrite'               => array( 'slug' => 'ecw-cart-links' ),
        );

        $args = apply_filters( 'ecw_shopping_cart_cpt_args', $args );

        register_post_type( ECW_CPT_NAME, $args );

    }

    /**
     * Responsible for creating and updating cart link entry
     *
     * @param int|null $cartID
     *
     * @return int
     * @since 1.2.0
     */
    public function ecwCreateCartLinkEntry( $cartID ){

        // get user id if it's logged in
        $customerID = ( is_user_logged_in() ) ? get_current_user_id() : 'unregistered';

        // insert new post
        $insertPost = array(
                    'post_type'         => ECW_CPT_NAME,
                    'comment_status'    => 'closed',
                    'ping_status'       => 'closed',
                    'post_status'       => 'publish'
                );

        // Insert the post into the database
        $cartID = wp_insert_post( $insertPost );

        // Hash the post ID to be used as the cart name
        $hashedCartID = md5( $cartID );

        $updatePost = array(
            'ID'            => $cartID,
            'post_title'    => $hashedCartID,
            'post_name'     => $hashedCartID,
        );

        // Update post title and post name
        wp_update_post( $updatePost );

        // Update the post meta of the post
        update_post_meta( $cartID, '_cart_id', $hashedCartID );
        update_post_meta( $cartID, '_cart_customer_id', $customerID );
        update_post_meta( $cartID, '_cart_session_data', maybe_serialize( WC()->cart ) );

        // Set cron event for cart expiration
        $ecwCron = ECW_Cron::getInstance();
        $ecwCron->ecwSetCartExpiration( $cartID );

        do_action( 'ecw_create_cart_link_entry', $cartID );

        return $cartID;

    }

    /**
     * Set new Column for Shopping Cart CPT
     *
     * @param array $columns
     *
     * @return array
     * @since 1.0.0
     */
    public function ecwSetNewShoppingCartColumn( $columns ){

        $newColumns = array(    'ecw-cart-link'         => __( 'Shopping Cart URL', 'email-cart-for-woocommerce' ),
                                'ecw-cart-customer'     => __( 'Customer', 'email-cart-for-woocommerce' ),
                                'ecw-cart-expiry'       => __( 'Expiry Date', 'email-cart-for-woocommerce' ),
                                'ecw-actions'           => ''
                            );

        $newColumns = apply_filters( 'ecw_new_cart_columns', $newColumns );

        return array_merge( $columns, $newColumns );

    }

    /**
     * Set actions Column for Shopping Cart CPT
     *
     * @param string $columns
     * @param int $cartID
     *
     * @since 1.0.0
     * @since 1.2.0 On the new update we are now using post meta to store the cron args. Made adjustments on how we pull cron args for cart expiry column.
     */
    public function ecwShoppingCartNewColumns( $column, $cartID ){

        switch ( $column ) {

            case 'ecw-cart-link':

                $cartLink = '';

                if ( get_post_status( $cartID ) == 'publish' ) {

                    $cartLink = trailingslashit( wc_get_cart_url() );
                    $cartLink = $cartLink . '?cid=' . md5( $cartID );

                }

                echo '<input type="text" name="shopping_cart_link" value="' . $cartLink . '" readonly="readonly">';

                do_action( 'ecw_cart_link', $column, $cartID );

                break;

            case 'ecw-cart-customer':

                    $userID = get_post_meta( $cartID, '_cart_customer_id', true );

                    if( is_numeric( $userID ) ){

                        $user = get_user_by( 'id', $userID );
                        if( ! empty ( $user->data->display_name ) ){
                            echo '<label>' . __( 'User: ', 'email-cart-for-woocommerce' ) . '</label>';
                            echo '<a href="' . get_edit_user_link( $userID ) . '">';
                            echo $user->data->display_name;
                            echo '</a><br>';
                        }
                        if( ! empty( $user->data->user_email ) ){
                            echo '<label>' . __( 'Email: ', 'email-cart-for-woocommerce' ) . '</label>';
                            echo '<a href="mailto:' . $user->data->user_email . '">';
                            echo $user->data->user_email;
                            echo '</a><br>';
                        }
                        if( ! empty( $user->roles ) ){

                            echo '<label>' . __( 'Role: ', 'email-cart-for-woocommerce' ) . '</label>';
                            foreach ( $user->roles as $key => $value ) {
                                echo ucwords( str_replace( '_', ' ', $value ) ).'<br>';
                            }
                        }

                    }else{
                        _e( 'Unregistered', 'email-cart-for-woocommerce' );
                    }

                do_action( 'ecw_cart_customer', $column, $cartID );

                break;

            case 'ecw-cart-expiry' :

                $cartExpiryDate = $this->ecwGetCartExpirationDate( $cartID );

                echo apply_filters( 'ecw_cart_expiry_column', $cartExpiryDate, $column, $cartID );

                do_action( 'ecw_cart_expiry', $cartExpiryDate, $column, $cartID );

                break;

            case 'ecw-actions' :

                $cartLink = trailingslashit( wc_get_cart_url() );
                $cartLink = $cartLink . '?cid=' . md5( $cartID );

                echo '<a href="#" data-id="' . $cartID . '" data-nonce="' . wp_create_nonce( 'my_delete_post_nonce' ) . '" class="delete-cart ' . get_post_status( $cartID ) . '"><span class="dashicons dashicons-no"></span>' . __( 'Delete', 'email-cart-for-woocommerce' ) . '</a>';
                echo '&nbsp;&nbsp;';
                echo '<a class="copyLink" id="copy' . $cartID . '" data-clipboard-text="' . $cartLink . '" title="' . __( 'Copy to clipboard', 'email-cart-for-woocommerce' ) . '"><span class="dashicons dashicons-admin-page"></span>'. __( 'Copy Link', 'email-cart-for-woocommerce' ) . '</a>';

                do_action( 'ecw_cart_action', $column, $cartID );

                break;

        }

    }

    /**
     * Set Confirmation Delete HTML modal box
     *
     * @since 1.0.0
     */
    public function ecwSetConfirmDeleteItemBoxHTML() {

        $screen = get_current_screen();

        if( is_admin() && $screen->post_type == ECW_CPT_NAME ){

            do_action( 'ecw_before_delete_confirmation' );

            echo '<div id="dialog-message" title="Warning" style="display:none;">
                        <span class="ui-state-default"><span class="ui-icon ui-icon-info" style="float:left; margin:0 7px 0 0;"></span></span>
                        <div>
                            <p>' . __( 'This will remove the selected cart link and customers will no longer be able to restore it using this link. Are you sure you wish to delete it?', 'email-cart-for-woocommerce') . '</p>
                        </div>
                    </div>';

            do_action( 'ecw_after_delete_confirmation' );

        }

    }

     /**
     * Set cart object
     *
     * @param array $cid
     *
     * @return object
     * @since 1.0.0
     * @since 1.3.0 ECW-103
     */
    public function ecwSetCartItems( $cid ){

        $items          = maybe_unserialize( get_post_meta( $cid, '_cart_session_data', true ) );
        $couponCodes    = isset( $items->applied_coupons ) ? $items->applied_coupons : '';
        $ecwBundles     = ECW_Bundled_Products::getInstance();
        $ecwComposite   = ECW_Composite_Products::getInstance();

        $this->ecwWooCommerceInit();
        
        // We need this constant var for cart grand total calculation since WC only allow computation of
        // grand total if you are in the cart or checkout page
        if ( ! defined( 'WOOCOMMERCE_CART' ) )
            define( 'WOOCOMMERCE_CART', true );

        // Empty cart and coupon first
        WC()->cart->empty_cart();
        WC()->cart->remove_coupons();

        if( ! empty( $items ) ){
            foreach ( $items->cart_contents as $itemID => $item ){

                $productID      = ! empty( $item[ 'variation_id' ] ) ? $item[ 'variation_id' ] : $item[ 'product_id' ];
                $variationID    = ! empty( $item[ 'variation_id' ] ) ? $item[ 'variation_id' ] : '';
                $quantity       = ! empty( $item[ 'quantity' ] ) ? $item[ 'quantity' ] : '';
                $variation      = ! empty( $item[ 'variation' ] ) ? $item[ 'variation' ] : array();
                $product        = wc_get_product( $productID );
                $itemData       = WC()->cart->get_item_data( $item );
                $itemData       = !empty( $itemData ) ? $itemData : array();

                if ( $product->is_type( 'bundle' ) && ECW_Bundled_Products::ecwCheckIfBundledParent( $item ) ){

                    // Set http request variables
                    $ecwBundles->ecwSetBundledRequestVariables( $item, $productID, $quantity, $product );

                    // Add bundled product
                    WC()->cart->add_to_cart( $productID, $quantity, $variationID, $variation, $itemData );

                } elseif ( $product->is_type( 'composite' ) && ECW_Composite_Products::ecwCheckIfCompositeParent( $item ) ){

                    // Set http request variables
                    $ecwComposite->ecwSetCompositeRequestVariables( $item, $productID, $quantity, $product );

                    // Add composite product
                    WC()->cart->add_to_cart( $productID, $quantity, $variationID, $variation, $itemData );

                } else if( empty( $item[ 'bundled_by' ] ) && empty( $item[ 'composite_parent' ] ) ) // Only add product if the item is not under bundle or composite product
                    WC()->cart->add_to_cart( $productID, $quantity, $variationID, $variation, $itemData );

            }
        }

        // apply coupons
        if( ! empty( $couponCodes ) ){
            foreach ( $couponCodes as $code ) {

                // check if coupon is applied or not, if not apply it
                if ( ! WC()->cart->has_discount( $code ) )
                    WC()->cart->add_discount( $code );

            }
        }

        return WC()->cart;

    }

    /**
     * Populate shopping cart with data using the link sent by the sender.
     *
     * @since 1.0.0
     * @since 1.3.0 ECW-103
     */
    public function ecwPopulateCart(){

        global $wpdb, $woocommerce_composite_products;
        $ecwBundles     = ECW_Bundled_Products::getInstance();
        $ecwComposite   = ECW_Composite_Products::getInstance();

        if( isset( $_GET[ 'cid' ] ) ){

            $cid = esc_sql( $_GET[ 'cid' ] );

            $cartExist = $wpdb->get_results(
                $wpdb->prepare(
                        apply_filters( "ecw_cart_exist_query",
                            "SELECT post_id FROM $wpdb->postmeta
                             WHERE meta_key = '_cart_id'
                             AND meta_value = %s
                             LIMIT 1
                            " ),
                        $cid
                    )
            );

            // Redirect if cart is not found in the db otherwise continue with the restoration
            if( empty( $cartExist ) ){
                wp_redirect( trailingslashit( wc_get_cart_url() ), 301 ); exit;
            }

            // Empty cart and coupon first
            WC()->cart->empty_cart();
            WC()->cart->remove_coupons();

            $cartData = get_post_meta( $cartExist[ 0 ]->post_id, '_cart_session_data', true );

            if( ! empty( $cartData ) ){

                $cart           = maybe_unserialize( $cartData );
                $couponCodes    = $cart->applied_coupons;
                $cartItems      = $cart->cart_contents;

                // Set cart ID session on frontend
                if ( ! is_admin() )
                    WC()->session->set( 'ecw_cart_id', $cartExist[ 0 ]->post_id );

                // add items into cart
                foreach ( $cartItems as $itemID => $item ) {

                    $productID      = ! empty( $item[ 'variation_id' ] ) ? $item[ 'variation_id' ] : $item[ 'product_id' ];
                    $variationID    = ! empty( $item[ 'variation_id' ] ) ? $item[ 'variation_id' ] : '';
                    $quantity       = ! empty( $item[ 'quantity' ] ) ? $item[ 'quantity' ] : '';
                    $variation      = ! empty( $item[ 'variation' ] ) ? $item[ 'variation' ] : array(); 
                    $product        = wc_get_product( $productID );
                    $itemData       = WC()->cart->get_item_data( $item );
                    $itemData       = !empty( $itemData ) ? $itemData : array();

                    if ( $product->is_type( 'bundle' ) && ECW_Bundled_Products::ecwCheckIfBundledParent( $item ) ){

                        // Set http request variables
                        $ecwBundles->ecwSetBundledRequestVariables( $item, $productID, $quantity, $product );

                        // Add bundled product
                        WC()->cart->add_to_cart( $productID, $quantity, $variationID, $variation, $itemData );

                    } elseif ( $product->is_type( 'composite' ) && ECW_Composite_Products::ecwCheckIfCompositeParent( $item ) ) {

                        // Set http request variables
                        $ecwComposite->ecwSetCompositeRequestVariables( $item, $productID, $quantity, $product );

                        // Add composite product
                        WC()->cart->add_to_cart( $productID, $quantity, $variationID, $variation, $itemData );

                    } else if( empty( $item[ 'bundled_by' ] ) && empty( $item[ 'composite_parent' ] ) ) // Only add product if the item is not under bundle or composite product
                        WC()->cart->add_to_cart( $productID, $quantity, $variationID, $variation, $itemData );

                    do_action( 'ecw_cart_restore', $productID, $quantity, $variationID, $variation, $itemData );

                }

                // apply coupons
                if( apply_filters( 'ecw_has_coupon_discount', true ) ){
                    foreach ( $couponCodes as $code ) {

                        // check if coupon is applied or not, if not apply it
                        if ( ! WC()->cart->has_discount( $code ) )
                            WC()->cart->add_discount( $code );

                    }
                }

                do_action( 'ecw_after_cart_restore' );

                // Redirect after all items are added to cart
                $cartURL = trailingslashit( wc_get_cart_url() );
                wp_redirect( apply_filters( 'ecw_redirect_url', $cartURL ), 301 ); exit;

            }
        }
    }

    /**
     * Get cart expiration date.
     *
     * @param int $cartID
     *
     * @since 1.3.0
     */
    public function ecwGetCartExpirationDate( $cartID ){

        $cartExpArgs    = get_post_meta( $cartID, ECW_CART_EXPIRATION_CRON_ARGS, true );
        $timestamp      = ! empty( $cartExpArgs ) ? wp_next_scheduled( ECW_CRON_HOOK, $cartExpArgs ) : false;

        if( $timestamp )
            $expirationDate = get_date_from_gmt( date( 'Y-m-d H:i:s', $timestamp ), 'M j, Y @ h:i A' );
        else
            $expirationDate = __( 'No Expiry', 'email-cart-for-woocommerce' );

        return apply_filters( 'ecw_expiration_date', $expirationDate, $cartID );

    }

    /**
     * Log user cart restores upon order process
     *
     * @param int $orderID  WC_Order object ID
     *
     * @since 1.4.0
     */
    public function ecwLogUserCartRestore( $orderID ) {

        $order   = wc_get_order( $orderID );
    	$cartID  = WC()->session->get( 'ecw_cart_id' );
        $user_id = ECW_Helper::get_order_data( $order , 'customer_user' );

    	$restore_logs = get_post_meta( $cartID , ECW_CART_RESTORE_LOGS , true );
    	$restore_logs = ! empty( $restore_logs ) ? $restore_logs : array();

    	end( $restore_logs );
    	$logID = intval( key( $restore_logs ) ) + 1;

    	// get user role
    	if ( $user_id > 0 ) {
    		$user = get_userdata( $user_id );
    		$role = implode( ', ' , $user->roles );
    	} else {
    		$role = 'Guest';
    	}

        $first_name = ECW_Helper::get_order_data( $order , 'billing_first_name' );
        $last_name  = ECW_Helper::get_order_data( $order , 'billing_last_name' );

    	$restore_logs[ $logID ] = array(
    		'customer'      =>  $first_name . ' ' . $last_name,
    		'email'         =>  ECW_Helper::get_order_data( $order , 'billing_email' ),
    		'role'          =>  $role,
    		'userID'        =>  ECW_Helper::get_order_data( $order , 'customer_user' ),
    		'orderID'       =>  $orderID,
    		'Date'          =>  ECW_Helper::get_order_data( $order , 'order_date' ),
    	);

    	update_post_meta( $cartID , ECW_CART_RESTORE_LOGS , $restore_logs );

        // clear cartID on session
        WC()->session->set( 'ecw_cart_id', false );
    }

    /**
     * save emal cart form information to database
     *
     * @param array $response   ecw_send_email response details
     *
     * @since 1.4.0
     */
    public function ecwSaveCartFormInfo( $response ) {

        if ( $response['status'] != 'success' || ! isset( $response[ 'cartID' ] ) )
            return;

        update_post_meta( $response['cartID'] , 'ecw_cart_form_info' , array(
            'sendersName'       => $response['sendersName'],
            'recipientName'     => $response['recipientName'],
            'recipientEmail'    => $response['recipientEmail'],
            'subject'           => $response['subject'],
            'message'           => $response['message']
        ) );
    }
}
